@@ -27,8 +27,6 @@
 #include <QtGui>
 #include <QtCore>
 #include <QTabBar>
-#include <QMenu>
-#include "../src/parameters.hpp"
 
 class MyTabBar : public QTabBar
 {
@@ -40,11 +38,13 @@ public:
 
 signals:
 	void swapTabs(int, int);
+	void toggledEnable(int, bool);
 
 public slots:
 	// void dragDropChange();
 	void slotMoveLeft();
 	void slotMoveRight();
+	void slotToggleActive(int state);
 
 protected:
 	void mouseReleaseEvent(QMouseEvent *event);
