@@ -20,23 +20,22 @@
  * Authors: Krzysztof Marczak (buddhi1980@gmail.com)
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <math.h>
 #include "cimage.hpp"
+#include <qpainter.h>
+#include <QtCore>
 
-#include <iostream>
-#include <new>
-#include <QtGui>
-
-cImage::cImage(int w, int h, bool low_mem)
+cImage::cImage(int w, int h, bool _allocLater)
 {
+	isAllocated = false;
 	width = w;
 	height = h;
-	previewAllocated = false;
-	lowMem = low_mem;
+	allocLater = _allocLater;
+	construct();
+}
 
+void cImage::construct()
+{
+	previewAllocated = false;
 	image8 = NULL;
 	image16 = NULL;
 	imageFloat = NULL;
@@ -45,9 +44,14 @@ cImage::cImage(int w, int h, bool low_mem)
 	opacityBuffer = NULL;
 	colourBuffer = NULL;
 	zBuffer = NULL;
+	gammaTable = NULL;
+
+	// optional image buffers
+	normalFloat = NULL;
+	normal8 = NULL;
+	normal16 = NULL;
 
 	AllocMem();
-	gammaTable = new int[65536];
 	progressiveFactor = 1;
 	imageWidget = NULL;
 	isUsed = false;
@@ -57,15 +61,17 @@ cImage::cImage(int w, int h, bool low_mem)
 	previewWidth = 0;
 	previewHeight = 0;
 
+	gammaTablePrepared = false;
+
 	isMainImage = false;
 }
 
 cImage::~cImage()
 {
+	isAllocated = false;
 	//qDebug() << "cImage::~cImage()";
 	FreeImage();
-	if (gammaTable) delete[] gammaTable;
-	gammaTable = NULL;
+
 	if (previewAllocated)
 	{
 		if (preview) delete[] preview;
@@ -77,28 +83,39 @@ cImage::~cImage()
 
 bool cImage::AllocMem(void)
 {
+	isAllocated = false;
 	//qDebug() << "bool cImage::AllocMem(void)";
 	if (width > 0 && height > 0)
 	{
-		try
+		if (!allocLater)
 		{
-			imageFloat = new sRGBfloat[width * height];
-			image16 = new sRGB16[width * height];
-			image8 = new sRGB8[width * height];
-			zBuffer = new float[width * height];
-			alphaBuffer8 = new unsigned char[width * height];
-			alphaBuffer16 = new unsigned short[width * height];
-			opacityBuffer = new unsigned short[width * height];
-			colourBuffer = new sRGB8[width * height];
-			ClearImage();
-		} catch (std::bad_alloc& ba)
-		{
-			width = 0;
-			height = 0;
-			FreeImage();
-			qCritical() << "bad_alloc caught in cimage: " << ba.what()
-					<< ", maybe required image dimension to big?";
-			return false;
+			try
+			{
+				imageFloat = new sRGBfloat[width * height];
+				image16 = new sRGB16[width * height];
+				image8 = new sRGB8[width * height];
+				zBuffer = new float[width * height];
+				alphaBuffer8 = new unsigned char[width * height];
+				alphaBuffer16 = new unsigned short[width * height];
+				opacityBuffer = new unsigned short[width * height];
+				colourBuffer = new sRGB8[width * height];
+				if (opt.optionalNormal)
+				{
+					normalFloat = new sRGBfloat[width * height];
+					normal16 = new sRGB16[width * height];
+					normal8 = new sRGB8[width * height];
+				}
+				ClearImage();
+			}
+			catch (std::bad_alloc &ba)
+			{
+				width = 0;
+				height = 0;
+				FreeImage();
+				qCritical() << "bad_alloc caught in cimage: " << ba.what()
+										<< ", maybe required image dimension to big?";
+				return false;
+			}
 		}
 	}
 	else
@@ -115,16 +132,19 @@ bool cImage::AllocMem(void)
 
 	preview = NULL;
 	preview2 = NULL;
+	isAllocated = true;
 	return true;
 }
 
-bool cImage::ChangeSize(int w, int h)
+bool cImage::ChangeSize(int w, int h, sImageOptional optional)
 {
-	if (w != width || h != height)
+	if (w != width || h != height || !(optional == *GetImageOptional()) || allocLater)
 	{
 		width = w;
 		height = h;
+		SetImageOptional(optional);
 		FreeImage();
+		allocLater = false;
 		return AllocMem();
 	}
 	return true;
@@ -139,13 +159,22 @@ void cImage::ClearImage(void)
 	memset(alphaBuffer16, 0, (unsigned long int) sizeof(unsigned short) * width * height);
 	memset(opacityBuffer, 0, (unsigned long int) sizeof(unsigned short) * width * height);
 	memset(colourBuffer, 0, (unsigned long int) sizeof(sRGB8) * width * height);
-
+	if(opt.optionalNormal)
+	{
+		if(normalFloat)
+			memset(normalFloat, 0, (unsigned long int) sizeof(sRGBfloat) * width * height);
+		if(normal16)
+			memset(normal16, 0, (unsigned long int) sizeof(sRGB16) * width * height);
+		if(normal8)
+			memset(normal8, 0, (unsigned long int) sizeof(sRGB8) * width * height);
+	}
 	for (long int i = 0; i < width * height; ++i)
 		zBuffer[i] = 1e20;
 }
 
 void cImage::FreeImage(void)
 {
+	isAllocated = false;
 	//qDebug() << "void cImage::FreeImage(void)";
 	if (imageFloat) delete[] imageFloat;
 	imageFloat = NULL;
@@ -163,11 +192,20 @@ void cImage::FreeImage(void)
 	colourBuffer = NULL;
 	if (zBuffer) delete[] zBuffer;
 	zBuffer = NULL;
+	if (normalFloat) delete[] normalFloat;
+	normalFloat = NULL;
+	if (normal16) delete[] normal16;
+	normal16 = NULL;
+	if (normal8) delete[] normal8;
+	normal8 = NULL;
+	if (gammaTable) delete[] gammaTable;
+	gammaTable = NULL;
+	gammaTablePrepared = false;
 }
 
 sRGB16 cImage::CalculatePixel(sRGBfloat pixel)
 {
-
+	CalculateGammaTable();
 	float R = pixel.R * adj.brightness;
 	float G = pixel.G * adj.brightness;
 	float B = pixel.B * adj.brightness;
@@ -208,9 +246,15 @@ sRGB16 cImage::CalculatePixel(sRGBfloat pixel)
 
 void cImage::CalculateGammaTable(void)
 {
-	for (int i = 0; i < 65536; i++)
+	if (!gammaTablePrepared)
 	{
-		gammaTable[i] = pow(i / 65536.0, 1.0 / adj.imageGamma) * 65535;
+		if (!gammaTable) gammaTable = new int[65536];
+
+		for (int i = 0; i < 65536; i++)
+		{
+			gammaTable[i] = pow(i / 65536.0, 1.0 / adj.imageGamma) * 65535;
+		}
+		gammaTablePrepared = true;
 	}
 }
 
@@ -247,8 +291,15 @@ int cImage::GetUsedMB(void)
 	long int image8Size = (long int) width * height * sizeof(sRGB8);
 	long int colorSize = (long int) width * height * sizeof(sRGB8);
 	long int opacitySize = (long int) width * height * sizeof(unsigned short);
-	mb = (long int) (zBufferSize + alphaSize16 + alphaSize8 + image16Size + image8Size
-			+ imageFloatSize + colorSize + opacitySize) / 1024 / 1024;
+	long int optionalSize = 0;
+	if(opt.optionalNormal)
+	{
+		optionalSize += (long int) width * height * sizeof(sRGBfloat);
+		optionalSize += (long int) width * height * sizeof(sRGB16);
+		optionalSize += (long int) width * height * sizeof(sRGB8);
+	}
+	mb = (zBufferSize + alphaSize16 + alphaSize8 + image16Size + image8Size
+		  + imageFloatSize + colorSize + opacitySize + optionalSize) / 1024 / 1024;
 
 	return mb;
 }
@@ -256,6 +307,7 @@ int cImage::GetUsedMB(void)
 void cImage::SetImageParameters(sImageAdjustments adjustments)
 {
 	adj = adjustments;
+	gammaTablePrepared = false;
 	CalculateGammaTable();
 }
 
@@ -267,8 +319,7 @@ unsigned char* cImage::ConvertTo8bit(void)
 		image8[i].G = image16[i].G / 256;
 		image8[i].B = image16[i].B / 256;
 	}
-	unsigned char* ptr = (unsigned char*) image8;
-	return ptr;
+	return (unsigned char*) image8;
 }
 
 unsigned char* cImage::ConvertAlphaTo8bit(void)
@@ -277,8 +328,31 @@ unsigned char* cImage::ConvertAlphaTo8bit(void)
 	{
 		alphaBuffer8[i] = alphaBuffer16[i] / 256;
 	}
-	unsigned char* ptr = (unsigned char*) alphaBuffer8;
-	return ptr;
+	return alphaBuffer8;
+}
+
+unsigned char* cImage::ConvertNormalto16Bit(void)
+{
+	if(!opt.optionalNormal) return NULL;
+	for (long int i = 0; i < width * height; i++)
+	{
+		normal16[i].R = normalFloat[i].R * 65535;
+		normal16[i].G = normalFloat[i].G * 65535;
+		normal16[i].B = normalFloat[i].B * 65535;
+	}
+	return (unsigned char*) normal16;
+}
+
+unsigned char* cImage::ConvertNormalto8Bit(void)
+{
+	if(!opt.optionalNormal) return NULL;
+	for (long int i = 0; i < width * height; i++)
+	{
+		normal8[i].R = normalFloat[i].R * 255;
+		normal8[i].G = normalFloat[i].G * 255;
+		normal8[i].B = normalFloat[i].B * 255;
+	}
+	return (unsigned char*) normal8;
 }
 
 sRGB8 cImage::Interpolation(float x, float y)
@@ -374,9 +448,9 @@ void cImage::UpdatePreview(QList<int> *list)
 					}
 				}
 
-#ifdef win32
-#pragma omp parallel for
-#endif
+//#ifndef WIN32
+//#pragma omp parallel for
+//#endif
 				for (int x = 0; x < w; x++)
 				{
 					int R = 0;
@@ -452,8 +526,7 @@ void cImage::RedrawInWidget(QWidget *qwidget)
 {
 	if (IsPreview())
 	{
-		QTextStream out(stdout);
-		//out << "Redraw" << endl;
+		//qDebug() << "Redraw" << endl;
 
 		QWidget *widget;
 		if (qwidget) widget = qwidget;
