@@ -21,10 +21,18 @@
  */
 
 #include "command_line_interface.hpp"
-#include "global_data.hpp"
-#include "settings.hpp"
+#include "system.hpp"
+#include "initparameters.hpp"
+#include "fractal_container.hpp"
 #include "headless.h"
 #include "error_message.hpp"
+#include "global_data.hpp"
+#include "queue.hpp"
+#include "netrender.hpp"
+#include "animation_frames.hpp"
+#include "keyframes.hpp"
+#include "settings.hpp"
+#include "test.hpp"
 
 cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 {
@@ -54,27 +62,27 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 		QCoreApplication::translate("main", "N"));
 
 	QCommandLineOption overrideOption(QStringList() << "O" << "override",
-		QCoreApplication::translate("main", "Overrides item '<KEY>' from settings file with new value '<VALUE>'.\n"
-			"Specify multiple KEY=VALUE pairs by separating with a '#' (KEY1=VALUE1#KEY2=VALUE2). Quote whole expression to avoid whitespace parsing issues\n"
-			"Override fractal parameter in the form 'fractal<N>_KEY=VALUE' with <N> as index of fractal"),
-		QCoreApplication::translate("main", "KEY=VALUE"));
+		QCoreApplication::translate("main", "<KEY=VALUE> overrides item '<KEY>' from settings file with new value '<VALUE>'.\n"
+			"Specify multiple KEY=VALUE pairs by separating with a '#': <KEY1=VALUE1#KEY2=VALUE2>. Quote whole expression to avoid whitespace parsing issues\n"
+			"Override fractal parameter in the form 'fractal<N>_KEY=VALUE' with <N> being index of fractal"),
+		QCoreApplication::translate("main", "..."));
 
 	QCommandLineOption listOption(QStringList() << "L" << "list",
 				QCoreApplication::translate("main", "Lists all possible parameters '<KEY>' with corresponding default value '<VALUE>'."));
 
 	QCommandLineOption formatOption(QStringList() << "f" << "format",
 		QCoreApplication::translate("main", "Image output format:\n"
-			"jpg - JPEG format (default)\n"
-			"png - PNG format\n"
-			"png16 - 16-bit PNG format\n"
-			"png16alpha - 16-bit PNG with alpha channel format\n"
-			"exr - EXR format\n"
-			"tiff - TIFF format"),
+			"  jpg - JPEG format (default)\n"
+			"  png - PNG format\n"
+			"  png16 - 16-bit PNG format\n"
+			"  png16alpha - 16-bit PNG with alpha channel format\n"
+			"  exr - EXR format\n"
+			"  tiff - TIFF format"),
 		QCoreApplication::translate("main", "FORMAT"));
 
 	QCommandLineOption resOption(QStringList() << "r" << "res",
-		QCoreApplication::translate("main", "Overrides image resolution."),
-		QCoreApplication::translate("main", "WIDTHxHEIGHT"));
+		QCoreApplication::translate("main", "Overrides image resolution. Specify as width and height separated by 'x'"),
+		QCoreApplication::translate("main", "WxH"));
 
 	QCommandLineOption fpkOption("fpk",
 		QCoreApplication::translate("main", "Overrides frames per key parameter."),
@@ -100,7 +108,13 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 		QCoreApplication::translate("main", "N"));
 
 	QCommandLineOption queueOption(QStringList() << "q" << "queue",
-		QCoreApplication::translate("main", "Renders all images from common queue."));
+								   QCoreApplication::translate("main", "Renders all images from common queue."));
+
+	QCommandLineOption testOption(QStringList() << "t" << "test",
+								   QCoreApplication::translate("main", "This will run testcases on the mandelbulber instance"));
+
+	QCommandLineOption voxelOption(QStringList() << "V" << "voxel",
+									 QCoreApplication::translate("main", "Renders the voxel volume in a stack of images."));
 
 	QCommandLineOption statsOption(QStringList() << "stats",
 		QCoreApplication::translate("main", "Shows statistics while rendering in CLI mode."));
@@ -120,7 +134,6 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 	parser.addOption(flightOption);
 	parser.addOption(startOption);
 	parser.addOption(endOption);
-	parser.addOption(overrideOption);
 	parser.addOption(listOption);
 	parser.addOption(formatOption);
 	parser.addOption(resOption);
@@ -130,6 +143,9 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 	parser.addOption(portOption);
 	parser.addOption(noColorOption);
 	parser.addOption(queueOption);
+	parser.addOption(testOption);
+	parser.addOption(voxelOption);
+	parser.addOption(overrideOption);
 	parser.addOption(statsOption);
 	parser.addOption(helpInputOption);
 
@@ -152,6 +168,8 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 	cliData.outputText = parser.value(outputOption);
 	cliData.listParameters = parser.isSet(listOption);
 	cliData.queue = parser.isSet(queueOption);
+	cliData.voxel = parser.isSet(voxelOption);
+	cliData.test = parser.isSet(testOption);
 	cliData.showInputHelp = parser.isSet(helpInputOption);
 	systemData.statsOnCLI = parser.isSet(statsOption);
 
@@ -163,16 +181,16 @@ cCommandLineInterface::cCommandLineInterface(QCoreApplication *qapplication)
 
 	if (cliData.listParameters) cliData.nogui = true;
 	if (cliData.queue) cliData.nogui = true;
-
+	if (cliData.test) cliData.nogui = true;
 	cliTODO = modeBootOnly;
 }
 
 cCommandLineInterface::~cCommandLineInterface()
 {
-	// TODO Auto-generated destructor stub
+
 }
 
-void cCommandLineInterface::ReadCLI(void)
+void cCommandLineInterface::ReadCLI()
 {
 	bool checkParse = true;
 	bool settingsSpecified = false;
@@ -194,6 +212,7 @@ void cCommandLineInterface::ReadCLI(void)
 	// list parameters only
 	if (cliData.listParameters)
 	{
+		InitMaterialParams(1, gPar);
 		QList<QString> listOfParameters = gPar->GetListOfParameters();
 		out
 				<< cHeadless::colorize("\nList of main parameters:\n",
@@ -226,6 +245,18 @@ void cCommandLineInterface::ReadCLI(void)
 		exit(0);
 	}
 
+	// run test cases
+	if (cliData.test) {
+		QStringList arguments = gApplication->arguments();
+		arguments.removeOne(QString("--test"));
+		arguments.removeOne(QString("t"));
+
+		int status = 0;
+		Test test;
+		status |= QTest::qExec(&test, arguments);
+		exit(status);
+	}
+
 	// check netrender server / client
 	if (cliData.server)
 	{
@@ -320,6 +351,7 @@ void cCommandLineInterface::ReadCLI(void)
 					parSettings.Decode(gPar, gParFractal, gAnimFrames, gKeyframes);
 					settingsSpecified = true;
 					systemData.lastSettingsFile = filename;
+					systemData.settingsLoadedFromCLI = true;
 				}
 				else
 				{
@@ -600,6 +632,29 @@ void cCommandLineInterface::ReadCLI(void)
 		}
 	}
 
+	//voxel export
+	if (cliData.voxel)
+	{
+		QString folderString = gPar->Get<QString>("voxel_image_path");
+		QDir folder(folderString);
+		if(!folder.exists())
+		{
+			cErrorMessage::showMessage(
+				QObject::tr(
+					"Cannot start voxel export. Specified folder (%1) does not exist.").arg(folderString),
+				cErrorMessage::errorMessage);
+
+			cErrorMessage::showMessage(
+				QObject::tr("End frame has to be greater than start frame which is %1")
+					.arg(gPar->Get<int>("flight_first_to_render")),
+				cErrorMessage::errorMessage);
+			parser.showHelp(cliErrorFlightEndFrameSmallerStartFrame);
+		}
+		cliTODO = modeVoxel;
+		cliData.nogui = true;
+		systemData.noGui = true;
+	}
+
 	//folder for animation frames
 	if (cliData.outputText != "" && cliTODO == modeFlight)
 	{
@@ -617,7 +672,7 @@ void cCommandLineInterface::ReadCLI(void)
 		parser.showHelp(cliErrorSettingsFileNotSpecified);
 	}
 
-	if (cliData.nogui && cliTODO != modeKeyframe && cliTODO != modeFlight && cliTODO != modeQueue)
+	if (cliData.nogui && cliTODO != modeKeyframe && cliTODO != modeFlight && cliTODO != modeQueue && cliTODO != modeVoxel)
 	{
 		//creating output filename if it's not specified
 		if (cliData.outputText == "")
@@ -665,6 +720,12 @@ void cCommandLineInterface::ProcessCLI(void)
 			gMainInterface->headless->RenderQueue();
 			break;
 		}
+		case modeVoxel:
+		{
+			gMainInterface->headless = new cHeadless();
+			gMainInterface->headless->RenderVoxel();
+			break;
+		}
 		case modeBootOnly:
 		{
 			// nothing to be done
