@@ -65,32 +65,34 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 
 	const cFractal *defaultFractal = fractals.GetFractal(fractalIndex);
 
-	sExtendedAux extendedAux[NUMBER_OF_FRACTALS];
-	int maxFractal = (in.forcedFormulaIndex >= 0) ? in.forcedFormulaIndex : fractals.GetMaxFractalIndex();
-	int minFractal = (in.forcedFormulaIndex >= 0) ? in.forcedFormulaIndex : 0;
-	for (int i = minFractal; i <= maxFractal; i++)
-	{
-		extendedAux[i].r_dz = 1.0;
-		extendedAux[i].r = r;
-		extendedAux[i].color = 1.0;
-		extendedAux[i].actualScale = fractals.GetFractal(i)->mandelbox.scale;
-		extendedAux[i].DE = 1.0;
-		extendedAux[i].c = c;
-		extendedAux[i].cw = 0;
-		extendedAux[i].newR = 1e+20;
-		extendedAux[i].axisBias = 1e+20;
-		extendedAux[i].orbitTraps = 1e+20;
-		extendedAux[i].transformSampling = 1e+20;
-	}
+	sExtendedAux extendedAux;
+
+	extendedAux.r_dz = 1.0;
+	extendedAux.r = r;
+	extendedAux.color = 1.0;
+	extendedAux.actualScale = fractals.GetFractal(fractalIndex)->mandelbox.scale;
+	extendedAux.DE = 1.0;
+	extendedAux.c = c;
+	extendedAux.cw = 0;
+  extendedAux.foldFactor = 0.0;
+  extendedAux.minRFactor = 0.0;
+  extendedAux.scaleFactor = 0.0;
+  //extendedAux.newR = 1e+20;
+  //extendedAux.axisBias = 1e+20;
+  //extendedAux.orbitTraps = 1e+20;
+  //extendedAux.transformSampling = 1e+20;
 
 	//main iteration loop
 	int i;
 	int sequence = 0;
-	int lastSequnce = 0;
+
+	CVector3 lastGoodZ;
+	CVector3 lastZ;
 
 	for (i = 0; i < in.maxN; i++)
 	{
-
+    lastGoodZ = lastZ;
+    lastZ = z;
 
 		//hybrid fractal sequence
 		if (in.forcedFormulaIndex >= 0)
@@ -102,24 +104,17 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 			sequence = fractals.GetSequence(i);
 		}
 
-		//for optimized DE calculation
-		if(fractals.UseOptimizedDE())
-		{
-			extendedAux[sequence] = extendedAux[lastSequnce];
-			lastSequnce = sequence;
-		}
-
 		//foldings
 		if (in.common.foldings.boxEnable)
 		{
-			BoxFolding(z, &in.common.foldings, extendedAux[sequence]);
+			BoxFolding(z, &in.common.foldings, extendedAux);
 			r = z.Length();
 		}
 
 		if (in.common.foldings.sphericalEnable)
 		{
-			extendedAux[sequence].r = r;
-		  SphericalFolding(z, &in.common.foldings, extendedAux[sequence]);
+			extendedAux.r = r;
+		  SphericalFolding(z, &in.common.foldings, extendedAux);
 			r = z.Length();
 		}
 
@@ -129,7 +124,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 		//temporary vector for weight function
 		CVector3 tempZ = z;
 
-		extendedAux[sequence].r = r;
+		extendedAux.r = r;
 
 		if (!fractals.IsHybrid() || fractals.GetWeight(sequence) > 0.0)
 		{
@@ -138,42 +133,42 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 			{
 				case mandelbulb:
 				{
-					MandelbulbIteration(z, fractal, extendedAux[sequence]);
+					MandelbulbIteration(z, fractal, extendedAux);
 					break;
 				}
 				case mandelbulb2:
 				{
-					Mandelbulb2Iteration(z, extendedAux[sequence]);
+					Mandelbulb2Iteration(z, extendedAux);
 					break;
 				}
 				case mandelbulb3:
 				{
-					Mandelbulb3Iteration(z, extendedAux[sequence]);
+					Mandelbulb3Iteration(z, extendedAux);
 					break;
 				}
 				case mandelbulb4:
 				{
-					Mandelbulb4Iteration(z, fractal, extendedAux[sequence]);
+					Mandelbulb4Iteration(z, fractal, extendedAux);
 					break;
 				}
 				case fast_mandelbulb_power2:
 				{
-					MandelbulbPower2Iteration(z, extendedAux[sequence]);
+					MandelbulbPower2Iteration(z, extendedAux);
 					break;
 				}
 				case xenodreambuie:
 				{
-					XenodreambuieIteration(z, fractal, extendedAux[sequence]);
+					XenodreambuieIteration(z, fractal, extendedAux);
 					break;
 				}
 				case mandelbox:
 				{
-					MandelboxIteration(z, fractal, extendedAux[sequence]);
+					MandelboxIteration(z, fractal, extendedAux);
 					break;
 				}
 				case smoothMandelbox:
 				{
-					SmoothMandelboxIteration(z, fractal, extendedAux[sequence]);
+					SmoothMandelboxIteration(z, fractal, extendedAux);
 					break;
 				}
 				case boxFoldBulbPow2:
@@ -183,37 +178,37 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				}
 				case menger_sponge:
 				{
-					MengerSpongeIteration(z, extendedAux[sequence]);
+					MengerSpongeIteration(z, extendedAux);
 					break;
 				}
 				case kaleidoscopicIFS:
 				{
-					KaleidoscopicIFSIteration(z, fractal, extendedAux[sequence]);
+					KaleidoscopicIFSIteration(z, fractal, extendedAux);
 					break;
 				}
 				case aexion:
 				{
-					AexionIteration(z, w, i, fractal, extendedAux[sequence]);
+					AexionIteration(z, w, i, fractal, extendedAux);
 					break;
 				}
 				case hypercomplex:
 				{
-					HypercomplexIteration(z, w, extendedAux[sequence]);
+					HypercomplexIteration(z, w, extendedAux);
 					break;
 				}
 				case quaternion:
 				{
-					QuaternionIteration(z, w, extendedAux[sequence]);
+					QuaternionIteration(z, w, extendedAux);
 					break;
 				}
 				case benesi:
 				{
-					BenesiIteration(z, c, extendedAux[sequence]);
+					BenesiIteration(z, c, extendedAux);
 					break;
 				}
 				case bristorbrot:
 				{
-					BristorbrotIteration(z, extendedAux[sequence]);
+					BristorbrotIteration(z, extendedAux);
 					break;
 				}
 				case ides:
@@ -228,7 +223,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				}
 				case buffalo:
 				{
-					BuffaloIteration(z, fractal, extendedAux[sequence]);
+					BuffaloIteration(z, fractal, extendedAux);
 					break;
 				}
 				case quickdudley:
@@ -253,57 +248,72 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				}
         case msltoeDonut:
 				{
-          MsltoeDonutIteration(z, fractal, extendedAux[sequence]);
+          MsltoeDonutIteration(z, fractal, extendedAux);
 					break;
 				}
         case msltoesym2Mod:
 				{
-          MsltoeSym2ModIteration(z, c, fractal, extendedAux[sequence]);
+          MsltoeSym2ModIteration(z, c, fractal, extendedAux);
 					break;
 				}
         case msltoesym3Mod:
         {
-          MsltoeSym3ModIteration(z, c, i, fractal, extendedAux[sequence]);
+          MsltoeSym3ModIteration(z, c, i, fractal, extendedAux);
           break;
         }
         case msltoesym3Mod2:
         {
-          MsltoeSym3Mod2Iteration(z, c, fractal, extendedAux[sequence]);
+          MsltoeSym3Mod2Iteration(z, c, fractal, extendedAux);
           break;
         }
         case msltoesym3Mod3:
         {
-          MsltoeSym3Mod3Iteration(z, c, i, fractal, extendedAux[sequence]);
+          MsltoeSym3Mod3Iteration(z, c, i, fractal, extendedAux);
           break;
         }
         case msltoesym4Mod:
         {
-          MsltoeSym4ModIteration(z, c, fractal, extendedAux[sequence]);
+          MsltoeSym4ModIteration(z, c, fractal, extendedAux);
+          break;
+        }
+        case msltoeToroidal:
+        {
+          MsltoeToroidalIteration(z, fractal, extendedAux);
+          break;
+        }
+        case msltoeToroidalMulti:
+        {
+          MsltoeToroidalMultiIteration(z, fractal, extendedAux);
           break;
         }
 				case generalizedFoldBox:
 				{
-					GeneralizedFoldBoxIteration(z, fractal, extendedAux[sequence]);
+					GeneralizedFoldBoxIteration(z, fractal, extendedAux);
 					break;
 				}
         case aboxMod1:
         {
-          AboxMod1Iteration(z, fractal, extendedAux[sequence]);
+          AboxMod1Iteration(z, fractal, extendedAux);
           break;
         }
         case aboxMod2:
         {
-          AboxMod2Iteration(z, fractal, extendedAux[sequence]);
+          AboxMod2Iteration(z, fractal, extendedAux);
           break;
         }
         case aboxModKali:
         {
-          AboxModKaliIteration(z, fractal, extendedAux[sequence]);
+          AboxModKaliIteration(z, fractal, extendedAux);
+          break;
+        }
+        case aboxModKaliEiffie:
+        {
+          AboxModKaliEiffieIteration(z, c, i, fractal, extendedAux);
           break;
         }
         case aboxVSIcen1:
         {
-          AboxVSIcen1Iteration(z, c, fractal, extendedAux[sequence]);
+          AboxVSIcen1Iteration(z, c, fractal, extendedAux);
           break;
         }
         case aexionOctopusMod:
@@ -313,57 +323,102 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case amazingSurf:
         {
-          AmazingSurfIteration(z, fractal, extendedAux[sequence]);
+          AmazingSurfIteration(z, fractal, extendedAux);
           break;
         }
         case amazingSurfMod1:
         {
-          AmazingSurfMod1Iteration(z, fractal, extendedAux[sequence]);
+          AmazingSurfMod1Iteration(z, fractal, extendedAux);
+          break;
+        }
+        case amazingSurfMulti:
+        {
+          AmazingSurfMultiIteration(z, i, fractal, extendedAux);
           break;
         }
         case benesiPineTree:
         {
-          BenesiPineTreeIteration(z, c, fractal, extendedAux[sequence]);
+          BenesiPineTreeIteration(z, c, fractal, extendedAux);
           break;
         }
         case benesiT1PineTree:
         {
-          BenesiT1PineTreeIteration(z, c, i, fractal, extendedAux[sequence]);
+          BenesiT1PineTreeIteration(z, c, i, fractal, extendedAux);
+          break;
+        }
+        case collatz:
+        {
+          CollatzIteration(z, extendedAux);
+          break;
+        }
+        case collatzMod:
+        {
+          CollatzModIteration(z, c, fractal, extendedAux);
           break;
         }
         case eiffieMsltoe:
         {
-          EiffieMsltoeIteration(z, c, fractal, extendedAux[sequence]);
+          EiffieMsltoeIteration(z, c, fractal, extendedAux);
           break;
         }
         case foldBoxMod1:
         {
-          FoldBoxMod1Iteration(z, i, fractal, extendedAux[sequence]);
+          FoldBoxMod1Iteration(z, i, fractal, extendedAux);
           break;
         }
         case iqBulb:
         {
-          IQbulbIteration(z, fractal, extendedAux[sequence]);
+          IQbulbIteration(z, fractal, extendedAux);
           break;
         }
         case kalisets1:
         {
-          Kalisets1Iteration(z, c, fractal, extendedAux[sequence]);
+          Kalisets1Iteration(z, c, fractal, extendedAux);
+          break;
+        }
+        case mandelboxMenger:
+        {
+          MandelboxMengerIteration(z, c, i, fractal, extendedAux);
+          break;
+        }
+        case mandelbulbBermarte:
+        {
+          MandelbulbBermarteIteration(z, fractal, extendedAux);
+          break;
+        }
+        case mandelbulbKali:
+        {
+          MandelbulbKaliIteration(z, fractal, extendedAux);
+          break;
+        }
+        case mandelbulbKaliMulti:
+        {
+          MandelbulbKaliMultiIteration(z, c, fractal, extendedAux);
           break;
         }
         case mandelbulbMulti:
         {
-          MandelbulbMultiIteration(z, fractal, extendedAux[sequence]);
+          MandelbulbMultiIteration(z, c, fractal, extendedAux);
           break;
         }
         case mandelbulbVaryPowerV1:
         {
-          MandelbulbVaryPowerV1Iteration(z, i, fractal, extendedAux[sequence]);
+          MandelbulbVaryPowerV1Iteration(z, i, fractal, extendedAux);
           break;
         }
         case mengerMod1:
         {
-          MengerMod1Iteration(z, i, fractal, extendedAux[sequence]);
+          MengerMod1Iteration(z, i, fractal, extendedAux);
+          break;
+        }
+        case mengerMiddleMod:
+        {
+          MengerMiddleModIteration(z, c, i, fractal, extendedAux);
+          break;
+        }
+        case mengerPwr2Poly:
+        {
+          MengerPwr2PolyIteration(z, c, i, fractal, extendedAux);
           break;
         }
         case riemannSphereMsltoe:
@@ -376,9 +431,19 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
           RiemannSphereMsltoeV1Iteration(z, fractal);
           break;
         }
+        case riemannBulbMsltoeMod2:
+        {
+          RiemannBulbMsltoeMod2Iteration(z, fractal);
+          break;
+        }
         case quaternion3D:
         {
-          Quaternion3DIteration(z, fractal, extendedAux[sequence]);
+          Quaternion3DIteration(z, fractal, extendedAux);
+          break;
+        }
+        case fastImagscaPower2:
+        {
+          FastImagscaPower2Iteration(z);
           break;
         }
 
@@ -422,17 +487,17 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case transfBenesiT1:
         {
-          TransformBenesiT1Iteration(z, fractal, extendedAux[sequence]);
+          TransformBenesiT1Iteration(z, fractal, extendedAux);
           break;
         }
         case transfBenesiT1Mod:
         {
-          TransformBenesiT1ModIteration(z, fractal, extendedAux[sequence]);
+          TransformBenesiT1ModIteration(z, fractal, extendedAux);
           break;
         }
         case transfBenesiT2:
         {
-          TransformBenesiT2Iteration(z, fractal, extendedAux[sequence]);
+          TransformBenesiT2Iteration(z, fractal, extendedAux);
           break;
         }
         case transfBenesiT3:
@@ -472,18 +537,18 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case transfBoxFold:
         {
-          TransformBoxFoldIteration(z, fractal, extendedAux[sequence]);
+          TransformBoxFoldIteration(z, fractal, extendedAux);
           break;
         }
         case transfBoxFoldXYZ:
         {
-          TransformBoxFoldXYZIteration(z, fractal, extendedAux[sequence]);
+          TransformBoxFoldXYZIteration(z, fractal, extendedAux);
           break;
         }
 
         case transfBoxOffset:
         {
-          TransformBoxOffsetIteration(z, fractal);
+          TransformBoxOffsetIteration(z, fractal, extendedAux);
           break;
         }
         case transfFabsAddConstant:
@@ -513,7 +578,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case transfMultipleAngle:
         {
-          TransformMultipleAngle(z, fractal, extendedAux[sequence]);
+          TransformMultipleAngle(z, fractal, extendedAux);
           break;
         }
         case transfNegFabsAddConstant:
@@ -521,6 +586,11 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
           TransformNegFabsAddConstantIteration(z, fractal);
           break;
         }
+        case transfPwr2Polynomial:
+        {
+          TransformPwr2PolynomialIteration(z, fractal, extendedAux);
+          break;
+        }
         case transfRotation:
         {
         	TransformRotationIteration(z, fractal);
@@ -533,17 +603,17 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case transfScale:
         {
-        	TransformScaleIteration(z, fractal, extendedAux[sequence]);
+        	TransformScaleIteration(z, fractal, extendedAux);
         	break;
         }
         case transfScaleVaryV1:
         {
-          TransformScaleVaryV1Iteration(z, i, fractal, extendedAux[sequence]);
+          TransformScaleVaryV1Iteration(z, i, fractal, extendedAux);
           break;
         }
         case transfScale3D:
         {
-        	TransformScale3DIteration(z, fractal, extendedAux[sequence]);
+        	TransformScale3DIteration(z, fractal, extendedAux);
         	break;
         }
 				case platonicSolid:
@@ -553,7 +623,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				}
 				case transfRPower:
 				{
-					TransformPowerR(z, fractal, extendedAux[sequence]);
+					TransformPowerR(z, fractal, extendedAux);
 					break;
 				}
         case transfSphereInvC:
@@ -563,22 +633,52 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         }
         case transfSphericalOffset:
         {
-          TransformSphericalOffsetIteration(z, fractal, extendedAux[sequence]);
+          TransformSphericalOffsetIteration(z, fractal, extendedAux);
           break;
         }
         case transfSphericalFold:
         {
-          TransformSphericalFoldIteration(z, fractal, extendedAux[sequence]);
+          TransformSphericalFoldIteration(z, fractal, extendedAux);
+          break;
+        }
+        case transfSphericalFoldAbox:
+        {
+          TransformSphericalFoldAboxIteration(z, fractal, extendedAux);
+          break;
+        }
+        case transfSphericalFoldVaryV1:
+        {
+          TransformSphericalFoldVaryV1Iteration(z, i, fractal, extendedAux);
           break;
         }
         case transfSphericalPwrFold:
         {
-          TransformSphericalPwrFoldIteration(z, fractal, extendedAux[sequence]);
+          TransformSphericalPwrFoldIteration(z, fractal, extendedAux);
+          break;
+        }
+        case transfSurfFoldMulti:
+        {
+          TransformSurfFoldMultiIteration(z, fractal, extendedAux);
           break;
         }
         case transfZvectorAxisSwap:
         {
-          TransformZvectorAxisSwapIteration(z, fractal );
+					TransformZvectorAxisSwapIteration(z, fractal);
+          break;
+        }
+				case transfRotationFoldingPlane:
+				{
+					TransformRotationFoldingPlane(z, fractal, extendedAux);
+					break;
+				}
+        case transfQuaternionFold:
+        {
+          TransformQuaternionFoldIteration(z, c, fractal, extendedAux);
+          break;
+        }
+        case transfMengerFold:
+        {
+          TransformMengerFoldIteration(z, fractal, extendedAux);
           break;
         }
         // 4D  ---------------------------------------------------------------------------
@@ -593,7 +693,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         case mandelboxVaryScale4D:
         {
           CVector4 z4D(z, w);
-          MandelboxVaryScale4DIteration(z4D, i, fractal, extendedAux[sequence]);
+          MandelboxVaryScale4DIteration(z4D, i, fractal, extendedAux);
           z = z4D.GetXYZ();
           w = z4D.w;
           break;
@@ -609,7 +709,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         case transfBoxFold4D:
         {
           CVector4 z4D(z, w);
-          TransformBoxFold4DIteration(z4D, fractal, extendedAux[sequence]);
+          TransformBoxFold4DIteration(z4D, fractal, extendedAux);
           z = z4D.GetXYZ();
           w = z4D.w;
           break;
@@ -641,7 +741,7 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         case transfScale4D:
         {
           CVector4 z4D(z, w);
-          TransformScale4DIteration(z4D, fractal, extendedAux[sequence]);
+          TransformScale4DIteration(z4D, fractal, extendedAux);
           z = z4D.GetXYZ();
           w = z4D.w;
           break;
@@ -649,14 +749,15 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         case transfSphericalFold4D:
         {
           CVector4 z4D(z, w);
-          TransformSphericalFold4DIteration(z4D, fractal, extendedAux[sequence]);
+          TransformSphericalFold4DIteration(z4D, fractal, extendedAux);
           z = z4D.GetXYZ();
           w = z4D.w;
           break;
         }
 
 				default:
-					z = CVector3(0.0, 0.0, 0.0);
+					double high = fractals.GetBailout(sequence) * 10.0;
+					z = CVector3(high, high, high);
 					break;
 			}
 		}
@@ -702,8 +803,34 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 			z = SmoothCVector(tempZ, z, fractals.GetWeight(sequence));
 		}
 
-		//r calculation
-		r = sqrt(z.x * z.x + z.y * z.y + z.z * z.z + w * w);
+    //r calculation
+     //r = sqrt(z.x * z.x + z.y * z.y + z.z * z.z + w * w);
+    switch(fractal->formula)
+    {
+      case fastImagscaPower2:
+      {
+        CVector3 z2 = z * z;
+        r = sqrt(z2.x + z2.y + z2.z) + (z2.y *  z2.z) / (z2.x);
+        break;
+      }
+      //scator magnitudes
+      // magnitude in imaginary scator algebra
+
+      default:
+      {
+        r = sqrt(z.x * z.x + z.y * z.y + z.z * z.z + w * w);
+        break;
+      }
+    }
+
+    if(z.IsNotANumber())
+    {
+    	z = lastZ;
+    	r = z.Length();
+    	w = 0.0;
+    	out->maxiter = true;
+    	break;
+    }
 
 		//escape conditions
 		if (fractals.IsCheckForBailout(sequence))
@@ -731,35 +858,40 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 			else if (Mode == calcModeColouring)
 			{
 				double len = 0.0;
-				switch (in.common.fractalColoringAlgorithm)
+				switch (in.fractalColoring.coloringAlgorithm)
 				{
-					case fractalColoringStandard:
+					case sFractalColoring::fractalColoringStandard:
 					{
 						len = r;
 						break;
 					}
-					case fractalColoringZDotPoint:
+					case sFractalColoring::fractalColoringZDotPoint:
 					{
 						len = fabs(z.Dot(in.point));
 						break;
 					}
-					case fractalColoringSphere:
+					case sFractalColoring::fractalColoringSphere:
 					{
-						len = fabs((z - in.point).Length() - in.common.fractalColoringSphereRadius);
+						len = fabs((z - in.point).Length() - in.fractalColoring.sphereRadius);
 						break;
 					}
-					case fractalColoringCross:
+					case sFractalColoring::fractalColoringCross:
 					{
 						len = dMin(fabs(z.x), fabs(z.y), fabs(z.z));
 						break;
 					}
-					case fractalColoringLine:
+					case sFractalColoring::fractalColoringLine:
 					{
-						len = fabs(z.Dot(in.common.fractalColoringLineDirection));
+						len = fabs(z.Dot(in.fractalColoring.lineDirection));
+						break;
+					}
+					case sFractalColoring::fractalColoringNone:
+					{
+						len = r;
 						break;
 					}
 				}
-				if (fractal->formula != mandelbox || in.common.fractalColoringAlgorithm != fractalColoringStandard)
+				if (fractal->formula != mandelbox || in.fractalColoring.coloringAlgorithm != sFractalColoring::fractalColoringStandard)
 				{
 					if (len < minimumR) minimumR = len;
 				}
@@ -778,6 +910,19 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				}
 			}
 		}
+
+		if(z.IsNotANumber()) //detection of dead computation
+		{
+			//if(Mode != calcModeColouring)
+			//qWarning() << "Dead computation\n"
+			//		<< "iteration: " << i << "Formula:" << formula << "Sequence:" << sequence
+			//		<< "\nPoint:" << in.point.Debug()
+			//		<< "\nLast good z:" << lastGoodZ.Debug()
+			//		<< "\nPrevious z:" << lastZ.Debug();
+			z = lastGoodZ;
+			break;
+		}
+
 	}
 
 	//final calculations
@@ -785,15 +930,15 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 	{
 		if (fractals.IsHybrid())
 		{
-			if(extendedAux[sequence].r_dz > 0)
+			if(extendedAux.r_dz > 0)
 			{
 				if (fractals.GetDEFunctionType(0) == fractal::linearDEFunction)
 				{
-					out->distance = r / fabs(extendedAux[sequence].DE);
+					out->distance = r / fabs(extendedAux.DE);
 				}
 				else if (fractals.GetDEFunctionType(0) == fractal::logarithmicDEFunction)
 				{
-					out->distance = 0.5 * r * log(r) / extendedAux[sequence].r_dz;
+					out->distance = 0.5 * r * log(r) / extendedAux.r_dz;
 				}
 			}
 			else
@@ -818,6 +963,9 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 				case mandelbulb2:
 				case mandelbulb3:
 				case mandelbulb4:
+        case mandelbulbBermarte:
+        case mandelbulbKali:
+        case mandelbulbKaliMulti:
 				case mandelbulbMulti:
         case mandelbulbVaryPowerV1:
         case msltoesym2Mod:
@@ -825,41 +973,52 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
         case msltoesym3Mod2:
         case msltoesym3Mod3:
         case msltoesym4Mod:
+        case msltoeToroidal: // TO DO fix??
+        case msltoeToroidalMulti: // TO DO fix??
 				case quaternion:
-        case quaternion3D:
+        case transfQuaternionFold: // hmmm, this issue again
+				case quaternion3D:
 				case xenodreambuie:
 				{
-					if(extendedAux[sequence].r_dz > 0)
-						out->distance = 0.5 * r * log(r) / extendedAux[sequence].r_dz;
+					if(extendedAux.r_dz > 0)
+						out->distance = 0.5 * r * log(r) / extendedAux.r_dz;
 					else
 						out->distance = r;
 					break;
 				}
 				case mandelbox:
+        case mandelboxMenger:
 				case smoothMandelbox:
 				case mandelboxVaryScale4D:
 				case generalizedFoldBox:
         case foldBoxMod1:
 				case aboxModKali:
-        case mengerMod1:
+        case aboxModKaliEiffie:
 				case aboxMod1:
         case aboxMod2:
         case amazingSurf:
         case amazingSurfMod1:
+        case amazingSurfMulti:
         case kalisets1:
         case aboxVSIcen1:
         {
-					if(extendedAux[sequence].r_dz > 0)
-						out->distance = r / fabs(extendedAux[sequence].DE);
+					if(extendedAux.r_dz > 0)
+						out->distance = r / fabs(extendedAux.DE);
 					else
 						out->distance = r;
 					break;
         }
 				case kaleidoscopicIFS:
 				case menger_sponge:
+				case collatz:
+        case collatzMod:
+        case mengerMod1:
+        case mengerMiddleMod:
+        case transfMengerFold: // hmmm, this issue again
+        case mengerPwr2Poly:
         {
-					if(extendedAux[sequence].r_dz > 0)
-						out->distance = (r - 2.0) / (extendedAux[sequence].DE);
+					if(extendedAux.r_dz > 0)
+						out->distance = (r - 2.0) / (extendedAux.DE);
 					else
 						out->distance = r;
 					break;
@@ -880,50 +1039,71 @@ void Compute(const cNineFractals &fractals, const sFractalIn &in, sFractalOut *o
 
 			double mboxColor = 0.0;
 			double mboxDE = 1.0;
-			for (int h = minFractal; h <= maxFractal; h++)
-			{
-				mboxColor += extendedAux[h].color;
-				mboxDE *= extendedAux[h].DE; //mboxDE *= mandelboxAux[h].mboxDE + extendedAux[h].color;
-			}
+			mboxColor = extendedAux.color;
+			mboxDE = extendedAux.DE;
 
 			double r2 = r / fabs(mboxDE);
 			if (r2 > 20) r2 = 20;
 
 			if (mboxColor > 1000) mboxColor = 1000;
 
-			out->colorIndex = minimumR * 1000.0 + mboxColor * 100 + r2 * 5000.0;
+      out->colorIndex = minimumR * 1000.0 + mboxColor * 100 + r2 * 5000.0;
+     /* out->colorIndex = extendedAux.color * 100.0 * extendedAux.foldFactor  // folds part
+          + r * defaultFractal->mandelbox.color.factorR / 1e13 // abs z part
+          + extendedAux.scaleFactor * r * i / 1e15  // scale part conditional on i & r
+          + ((in.fractalColoring.coloringAlgorithm != sFractalColoring::fractalColoringStandard) ? minimumR
+              * extendedAux.minRFactor * 1000.0 :
+              0.0);*/
 		}
 		else
 		{
 			switch (formula)
 			{
 				case mandelbox:
+        //case mandelboxMenger:
 				case smoothMandelbox:
 				case mandelboxVaryScale4D:
 				case generalizedFoldBox:
+        //case amazingSurfMulti:
+        case amazingSurfMod1:
 				case foldBoxMod1:
-					out->colorIndex = extendedAux[fractalIndex].color * 100.0
-							+ r * defaultFractal->mandelbox.color.factorR
-							+ ((in.common.fractalColoringAlgorithm != fractalColoringStandard) ? minimumR
+          out->colorIndex = extendedAux.color * 100.0   // folds part
+              + r * defaultFractal->mandelbox.color.factorR / 1e13 // abs z part
+							+ ((in.fractalColoring.coloringAlgorithm != sFractalColoring::fractalColoringStandard) ? minimumR
 									* 1000.0 :
 									0.0);
 					break;
 
         case mengerMod1:
         case aboxModKali:
-        case aboxMod1:
+				case aboxMod1:
 				case menger_sponge:
+				case collatz:
+        case collatzMod:
 				case kaleidoscopicIFS:
+        case mengerPwr2Poly:
+        case mengerMiddleMod:
 					out->colorIndex = minimumR * 1000.0;
 					break;
 
 				case amazingSurf:
-				case amazingSurfMod1:
-					out->colorIndex = minimumR * 200.0;
+          out->colorIndex = minimumR * 200.0;
 					break;
 
+        case amazingSurfMulti:
+        case mandelboxMenger:
+      //case amazingSurfMod1:
+
+          out->colorIndex = extendedAux.color * 100.0 * extendedAux.foldFactor  // folds part
+              + r * defaultFractal->mandelbox.color.factorR / 1e13 // abs z part
+              + extendedAux.scaleFactor * r * i / 1e15  // scale part conditional on i & r
+              + ((in.fractalColoring.coloringAlgorithm != sFractalColoring::fractalColoringStandard) ? minimumR
+                  * extendedAux.minRFactor * 1000.0 :
+                  0.0);
+          break;
+
 				case msltoeDonut:
-					out->colorIndex = extendedAux[fractalIndex].color * 2000.0 / i;
+					out->colorIndex = extendedAux.color * 2000.0 / i;
 					break;
 
 				default:
