@@ -61,15 +61,6 @@ enum enumDEMethod
 	forceDeltaDEMethod = 1,
 };
 
-enum enumFractalColoringAlgorithm
-{
-	fractalColoringStandard = 0,
-	fractalColoringZDotPoint = 1,
-	fractalColoringSphere = 2,
-	fractalColoringCross = 3,
-	fractalColoringLine = 4
-};
-
 enum enumOCLDEMode
 {
 	ocl_DEcalculated = 0, ocl_deltaDE = 1, ocl_noDE = 2
@@ -113,6 +104,9 @@ struct sExtendedAux
 	double axisBias;
 	double transformSampling;
 	double cw;
+  double foldFactor;
+  double minRFactor;
+  double scaleFactor;
 	CVector3 c;
 };
 
@@ -123,6 +117,10 @@ struct sFoldColor
 	double factorR;
 	double factorSp1;
 	double factorSp2;
+  double colorMin;
+  double compFold;
+  double compMinR;
+  double compScale;
 };
 
 struct sFractalGeneralizedFoldBox
@@ -252,26 +250,90 @@ struct sFractalMandelbulbMulti
   {
     acos, asin
   };
+
   enum multi_atanOratan2
   {
     atan, atan2
   };
+
   enum multi_OrderOfxyz
   {
     xyz, xzy, yxz, yzx, zxy, zyx
-  };
+	};
+
   multi_acosOrasin acosOrasin;
+	multi_acosOrasin acosOrasinA;
   multi_atanOratan2 atanOratan2;
+	multi_atanOratan2 atanOratan2A;
+
   multi_OrderOfxyz orderOfxyz;
+	multi_OrderOfxyz orderOfxyz2;
+	multi_OrderOfxyz orderOfxyzC;
 };
 
+//sinTan2Trig
+struct sFractalSinTan2Trig
+{
+  enum multi_asinOracos
+  {
+    asin, acos
+  };
+
+  enum multi_atan2Oratan
+  {
+    atan2, atan
+  };
+
+  enum multi_OrderOfzyx
+  {
+    zyx, zxy, yzx, yxz, xzy, xyz
+  };
+
+  multi_asinOracos asinOracos;
+  multi_atan2Oratan atan2Oratan;
+  multi_OrderOfzyx orderOfzyx;
+};
+
+//surf fold box
+struct sFractalSurfFolds
+{
+  enum multi_orderOfFolds
+  {
+    type1, type2, type3, type4, type5
+  };
+  multi_orderOfFolds orderOfFolds1;
+  multi_orderOfFolds orderOfFolds2;
+  multi_orderOfFolds orderOfFolds3;
+  multi_orderOfFolds orderOfFolds4;
+  multi_orderOfFolds orderOfFolds5;
+};
+
+struct sFractalAnalyticDE
+{
+  bool enabledFalse;
+  bool enabledAuxR2False;
+  double scale1;
+  double offset1;
+  double offset2;
+  double scaleLin;
+  double offsetLin;
+};
+
+
 //common parameters for transforming formulas
 struct sFractalTransformCommon
 {
-
+  double alphaAngleOffset;
+  double betaAngleOffset;
+  //double colorMin;
+  //double compFold;
+  //double compMinR;
+  //double compScale;
   double offset;
   double offset0;
   double offset05;
+  double offset1;
+  double offset105;
   double offset2;
   double multiplication;
   double minR0;
@@ -279,14 +341,22 @@ struct sFractalTransformCommon
   double minR06;
   double scale;
   double scale0;
+  double scale025;
   double scale1;
   double scaleA1;
+  double scaleB1;
+  double scaleC1;
   double scale015;
+  double scale2;
   double scale3;
+  double scale4;
+  double scale8;
   double pwr05;
   double pwr4;
   double pwr8;
   double pwr8a;
+  double sqtR;
+  double mboxFactor1;
 
   int startIterations;
   int startIterations250;
@@ -297,8 +367,22 @@ struct sFractalTransformCommon
   int stopIterationsB;
   int startIterationsC;
   int stopIterationsC;
+  int startIterationsD;
+  int stopIterationsD;
+  int startIterationsE;
+  int stopIterationsE;
+  int startIterationsR;
+  int stopIterationsR;
+  int startIterationsS;
+  int stopIterationsS;
+  int startIterationsT;
+  int stopIterationsT;
+  int startIterationsM;
+  int stopIterationsM;
+  int stopIterations1;
   int intA;
   int intB;
+  int int1;
 
   CVector3 additionConstant0555;
   CVector3 additionConstant000;
@@ -312,6 +396,9 @@ struct sFractalTransformCommon
   CVector3 constantMultiplier010;
   CVector3 constantMultiplier100;
   CVector3 constantMultiplier111;
+  CVector3 constantMultiplierA111;
+  CVector3 constantMultiplierB111;
+  CVector3 constantMultiplierC111;
   CVector3 constantMultiplier121;
   CVector3 constantMultiplier122;
   CVector3 constantMultiplier221;
@@ -322,7 +409,7 @@ struct sFractalTransformCommon
   CVector3 offset200;
   CVector3 power025;
   CVector3 power8;
-	CVector3 rotation;
+  CVector3 rotation;
   CVector3 scale3D111;
   CVector3 scale3D222;
   CVector3 scale3D333;
@@ -351,6 +438,9 @@ struct sFractalTransformCommon
   bool functionEnabledAxFalse;
   bool functionEnabledAyFalse;
   bool functionEnabledAzFalse;
+  bool functionEnabledBxFalse;
+  bool functionEnabledByFalse;
+  bool functionEnabledBzFalse;
   bool juliaMode;
   bool rotationEnabled;
 };
@@ -372,8 +462,12 @@ public:
 	sFractalBuffalo buffalo;
 	sFractalPlatonicSolid platonicSolid;
   sFractalTransformCommon transformCommon;
+  sFractalAnalyticDE analyticDE;
   sFractalMandelbulbMulti mandelbulbMulti;
+  sFractalSinTan2Trig sinTan2Trig;
+  sFractalSurfFolds surfFolds;
   sFractalDonut donut;
+  sFoldColor foldColor;
 
 #ifdef CLSUPPORT
 	double customParameters[15];
