@@ -42,7 +42,7 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					deltaDEType,
 																					withoutDEFunction,
 																					cpixelDisabledByDefault,
-																					100));
+																					1));
 
 	fractalList->append(sFractalDescription("AboxMod1",
 																					"aboxMod1",
@@ -68,6 +68,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					100));
 
+  fractalList->append(sFractalDescription("AboxModKaliEiffie",
+                                          "aboxModKaliEiffie",
+                                          aboxModKaliEiffie,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
   fractalList->append(sFractalDescription("AboxVS icen1",
                                           "aboxVS_icen1",
                                           aboxVSIcen1,
@@ -108,6 +116,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelEnabledByDefault,
                                           100));
 
+  fractalList->append(sFractalDescription("Amazing_Surf_Multi",
+                                          "amazing_surf_multi",
+                                          amazingSurfMulti,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelEnabledByDefault,
+                                          100));
+
 	fractalList->append(sFractalDescription("Benesi",
 																					"benesi",
 																					benesi,
@@ -156,6 +172,22 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Collatz Fractal",
+                                          "collatz",
+                                          collatz,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
+  fractalList->append(sFractalDescription("Collatz Fractal Mod ",
+                                          "collatz_mod",
+                                          collatzMod,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
   fractalList->append(sFractalDescription("Fold_Box_Mod1",
                                           "fold_box_mod1",
                                           foldBoxMod1,
@@ -244,6 +276,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					100));
 
+  fractalList->append(sFractalDescription("Mandelbox Menger",
+                                          "mandelbox_menger",
+                                          mandelboxMenger,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelEnabledByDefault,
+                                          100));
+
 	fractalList->append(sFractalDescription("Mandelbulb",
 																					"mandelbulb",
 																					mandelbulb,
@@ -276,6 +316,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					10));
 
+ fractalList->append(sFractalDescription("Mandelbulb_Bermarte",
+                                          "mandelbulb_bermarte",
+                                          mandelbulbBermarte,
+                                          analyticDEType,
+                                          logarithmicDEFunction,
+                                          cpixelEnabledByDefault,
+                                          10));
+
 	fractalList->append(sFractalDescription("Mandelbulb Classic Power 2",
 																					"mandelbulb_power_2",
 																					fast_mandelbulb_power2,
@@ -284,6 +332,22 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Mandelbulb_Kali",
+                                          "mandelbulb_kali",
+                                          mandelbulbKali,
+                                          analyticDEType,
+                                          logarithmicDEFunction,
+                                          cpixelEnabledByDefault,
+                                          10));
+
+  fractalList->append(sFractalDescription("Mandelbulb_Kali_Multi",
+                                          "mandelbulb_kali_multi",
+                                          mandelbulbKaliMulti,
+                                          analyticDEType,
+                                          logarithmicDEFunction,
+                                          cpixelEnabledByDefault,
+                                          10));
+
   fractalList->append(sFractalDescription("Mandelbulb_Multi",
 																					"mandelbulb_multi",
 																					mandelbulbMulti,
@@ -308,14 +372,31 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelDisabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Menger Pwr2 Polynomial",
+                                          "menger_pwr2_poly",
+                                          mengerPwr2Poly,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
   fractalList->append(sFractalDescription("Menger-Mod1",
                                           "menger_mod1",
                                           mengerMod1,
 																					analyticDEType,
 																					linearDEFunction,
-                                          cpixelAlreadyHas,
+                                          cpixelDisabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Menger Middle Mod",
+                                          "menger_middle_mod",
+                                          mengerMiddleMod,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
+
   fractalList->append(sFractalDescription("Msltoe Donut",
                                           "msltoe_donut",
 																					msltoeDonut,
@@ -372,6 +453,22 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelEnabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Msltoe Toroidal Bulb",
+                                          "msltoe_toroidal",
+                                          msltoeToroidal,
+                                          analyticDEType, // TO DO fix??
+                                          logarithmicDEFunction,
+                                          cpixelEnabledByDefault,
+                                          10));
+
+  fractalList->append(sFractalDescription("Msltoe Toroidal Multi",
+                                          "msltoe_toroidal_multi",
+                                          msltoeToroidalMulti,
+                                          analyticDEType, // TO DO fix??
+                                          logarithmicDEFunction,
+                                          cpixelEnabledByDefault,
+                                          10));
+
 	fractalList->append(sFractalDescription("Quaternion",
 																					"quaternion",
 																					quaternion,
@@ -420,6 +517,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelDisabledByDefault,
 																					10));
 
+  fractalList->append(sFractalDescription("Riemann Bulb Msltoe Mod2",
+                                          "riemann_bulb_msltoe_mod2",
+                                          riemannBulbMsltoeMod2,
+                                          deltaDEType,
+                                          logarithmicDEFunction,
+                                          cpixelDisabledByDefault,
+                                          1.5));
+
 	fractalList->append(sFractalDescription("Smooth Mandelbox",
 																					"smooth_mandelbox",
 																					smoothMandelbox,
@@ -436,6 +541,19 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelEnabledByDefault,
 																					10));
 
+	fractalList->append(sFractalDescription("Imaginary scator Power 2",
+																					"imagsca_power_2",
+																					fastImagscaPower2,
+																					deltaDEType,
+																					logarithmicDEFunction,
+																					cpixelEnabledByDefault,
+																					10));
+
+
+
+
+
+
   // ------------------ 4D -----------------------
   fractalList->append(sFractalDescription("Quaternion_4D",
                                           "quaternion_4D",
@@ -662,6 +780,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelAlreadyHas,
                                           100));
 
+  fractalList->append(sFractalDescription("Transform - Menger Fold",
+                                          "transf_menger_fold",
+                                          transfMengerFold,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
   fractalList->append(sFractalDescription("Transform - Multiple angle",
                                           "transf_multiple_angle",
                                           transfMultipleAngle,
@@ -678,13 +804,29 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelDisabledByDefault,
 																					100));
 
+  fractalList->append(sFractalDescription("Transform - Power2 Polynomial",
+                                          "transf_pwr2_polynomial",
+                                          transfPwr2Polynomial,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          10));
+
+  fractalList->append(sFractalDescription("Transform - Quaternion Fold",
+                                          "transf_quaternion_fold",
+                                          transfQuaternionFold,
+                                          analyticDEType,
+                                          logarithmicDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
 	fractalList->append(sFractalDescription("Transform - Rotation",
 																					"transf_rotation",
 																					transfRotation,
 																					analyticDEType,
 																					withoutDEFunction,
 																					cpixelDisabledByDefault,
-																					100));
+                                          100));
 
   fractalList->append(sFractalDescription("Transform - Rotation_VaryV1",
                                           "transf_rotation_varyV1",
@@ -694,6 +836,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelDisabledByDefault,
                                           100));
 
+  fractalList->append(sFractalDescription("Transform - Rotation Folding Plane",
+                                          "transf_rotation_folding_plane",
+                                          transfRotationFoldingPlane,
+                                          analyticDEType,
+                                          withoutDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
 	fractalList->append(sFractalDescription("Transform - R^Power",
 																					"transf_rpower",
 																					transfRPower,
@@ -742,6 +892,22 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
                                           cpixelDisabledByDefault,
                                           100));
 
+  fractalList->append(sFractalDescription("Transform - Spherical_Fold ABox",
+                                          "transf_spherical_fold_abox",
+                                          transfSphericalFoldAbox,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
+  fractalList->append(sFractalDescription("Transform - Spherical_Fold VaryV1",
+                                          "transf_spherical_fold_varyV1",
+                                          transfSphericalFoldVaryV1,
+                                          analyticDEType,
+                                          linearDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
   fractalList->append(sFractalDescription("Transform - Spherical_Pwr_Fold",
                                           "transf_spherical_pwr_fold",
                                           transfSphericalPwrFold,
@@ -758,6 +924,14 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelDisabledByDefault,
 																					100));
 
+  fractalList->append(sFractalDescription("Transform - Surf Fold Multi",
+                                          "transf_surf_fold_multi",
+                                          transfSurfFoldMulti,
+                                          analyticDEType,
+                                          withoutDEFunction,
+                                          cpixelDisabledByDefault,
+                                          100));
+
 	fractalList->append(sFractalDescription("Transform - Zvector_Axis_Swap",
 																					"transf_zVector_axis_swap",
 																					transfZvectorAxisSwap,
@@ -766,6 +940,8 @@ void DefineFractalList(QList<sFractalDescription> *fractalList)
 																					cpixelDisabledByDefault,
 																					100));
 
+
+
   fractalList->append(sFractalDescription("Transform - Addition Constant_4D",
                                           "transf_addition_constant_4D",
                                           transfAdditionConstant4D,
