@@ -25,12 +25,16 @@
 
 #include "fractparams.hpp"
 #include "parameters.hpp"
-#include "primitives.h"
+#include "object_types.hpp"
+
+extern cParameterContainer *gPar;
 
 void InitParams(cParameterContainer *par);
 void InitFractalParams(cParameterContainer *par);
 void InitPrimitiveParams(fractal::enumObjectType objectType, const QString primitiveName, cParameterContainer *par);
 void DeletePrimitiveParams(fractal::enumObjectType objectType, const QString primitiveName, cParameterContainer *par);
 void DeleteAllPrimitiveParams(cParameterContainer *par);
+void DeleteAllMaterialParams(cParameterContainer *par);
+void InitMaterialParams(int materialId, cParameterContainer *par);
 
 #endif /* INITPARAMETERS_HPP_ */
