@@ -21,13 +21,15 @@
  */
 
 #include "queue.hpp"
-#include "system.hpp"
 #include "settings.hpp"
 #include "preview_file_dialog.h"
-#include "global_data.hpp"
+#include "initparameters.hpp"
+#include "fractal_container.hpp"
 #include "error_message.hpp"
 #include "render_queue.hpp"
 
+cQueue *gQueue = NULL;
+
 cQueue::cQueue(cInterface *_interface, const QString &_queueListFileName,
 		const QString &_queueFolder, QObject *parent) :
 		QObject(parent), mainInterface(_interface)
@@ -36,7 +38,7 @@ cQueue::cQueue(cInterface *_interface, const QString &_queueListFileName,
 	queueListFileName = _queueListFileName;
 	queueFolder = _queueFolder;
 
-	if (!QFile::exists(queueFolder) && !CreateDirectory(queueFolder))
+	if (!QFile::exists(queueFolder) && !CreateFolder(queueFolder))
 	{
 		throw QString("cannot init queueListFileName folder to: " + queueFolder);
 	}
@@ -563,9 +565,9 @@ void cQueue::slotQueueRender()
 {
 	if (!systemData.noGui)
 	{
-		gMainInterface->SynchronizeInterfaceWindow(gMainInterface->mainWindow->ui->dockWidget_queue_dock,
+		SynchronizeInterfaceWindow(gMainInterface->mainWindow->ui->dockWidget_queue_dock,
 																							 gPar,
-																							 cInterface::read);
+																							 qInterface::read);
 	}
 	if (queueListFromFile.size() > 0)
 	{
@@ -581,7 +583,7 @@ void cQueue::slotQueueRender()
 
 void cQueue::slotQueueAddCurrentSettings()
 {
-	gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read); //update appParam before loading new settings
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read); //update appParam before loading new settings
 	Append();
 }
 
