@@ -25,11 +25,15 @@
 #include "settings.hpp"
 #include "old_settings.hpp"
 #include "preview_file_dialog.h"
+#include "file_image.hpp"
+#include "interface.hpp"
+#include "animation_flight.hpp"
+#include "animation_keyframes.hpp"
+#include "initparameters.hpp"
+#include "files.h"
 
 void RenderWindow::slotImportOldSettings()
 {
-  cSettings parSettings(cSettings::formatFullText);
-
   QFileDialog dialog(this);
   dialog.setOption(QFileDialog::DontUseNativeDialog);
   dialog.setFileMode(QFileDialog::ExistingFile);
@@ -47,7 +51,7 @@ void RenderWindow::slotImportOldSettings()
     oldSettings.LoadSettings(filename);
     oldSettings.ConvertToNewContainer(gPar, gParFractal);
     gMainInterface->RebuildPrimitives(gPar);
-    gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+    gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
     gMainInterface->ComboMouseClickUpdate();
     systemData.lastSettingsFile = filename;
     this->setWindowTitle(QString("Mandelbulber (") + filename + ")");
@@ -57,8 +61,8 @@ void RenderWindow::slotImportOldSettings()
 void RenderWindow::slotMenuAboutMandelbulber()
 {
   QString text = "<h2>Mandelbulber</h2>";
-  text += "version: <b>" + QString(MANDELBULBER_VERSION_STRING) + "</b>, build date: <b>"
-      + QString(__DATE__) + "</b><br>";
+  text += "version: <b>" + QString(MANDELBULBER_VERSION_STRING) + "</b>"
+      + "<br>";
   text += "<br>";
   text += "Licence: GNU GPL version 3.0<br>";
   text += "Copyright Ⓒ 2015<br>";
@@ -114,7 +118,8 @@ void RenderWindow::slotMenuLoadExample()
     parSettings.LoadFromFile(filename);
     parSettings.Decode(gPar, gParFractal, gAnimFrames, gKeyframes);
     gMainInterface->RebuildPrimitives(gPar);
-    gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+    gMainInterface->materialListModel->Regenerate();
+    gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
     gMainInterface->ComboMouseClickUpdate();
     systemData.lastSettingsFile = systemData.dataDirectory + "settings" + QDir::separator()
         + QFileInfo(filename).fileName();
@@ -126,7 +131,7 @@ void RenderWindow::slotMenuLoadExample()
 
 void RenderWindow::slotMenuLoadSettings()
 {
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read); //update appParam before loading new settings
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read); //update appParam before loading new settings
 
   cSettings parSettings(cSettings::formatFullText);
 
@@ -146,7 +151,8 @@ void RenderWindow::slotMenuLoadSettings()
     parSettings.LoadFromFile(filename);
     parSettings.Decode(gPar, gParFractal, gAnimFrames, gKeyframes);
     gMainInterface->RebuildPrimitives(gPar);
-    gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+    gMainInterface->materialListModel->Regenerate();
+    gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
     gMainInterface->ComboMouseClickUpdate();
     systemData.lastSettingsFile = filename;
     this->setWindowTitle(QString("Mandelbulber (") + filename + ")");
@@ -157,7 +163,7 @@ void RenderWindow::slotMenuLoadSettings()
 
 void RenderWindow::slotMenuLoadSettingsFromClipboard()
 {
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read); //update appParam before loading new settings
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read); //update appParam before loading new settings
 
   cSettings parSettings(cSettings::formatFullText);
 
@@ -165,7 +171,8 @@ void RenderWindow::slotMenuLoadSettingsFromClipboard()
   {
     parSettings.Decode(gPar, gParFractal, gAnimFrames, gKeyframes);
     gMainInterface->RebuildPrimitives(gPar);
-    gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+    gMainInterface->materialListModel->Regenerate();
+    gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
     gMainInterface->ComboMouseClickUpdate();
     systemData.lastSettingsFile = "from clipboard";
     this->setWindowTitle(QString("Mandelbulber (") + "from clipboard" + ")");
@@ -189,10 +196,17 @@ void RenderWindow::slotMenuResetDocksPositions()
 {
   restoreGeometry(defaultGeometry);
   restoreState(defaultState);
+  setCorner( Qt::TopLeftCorner, Qt::LeftDockWidgetArea );
+  setCorner( Qt::TopRightCorner, Qt::RightDockWidgetArea );
+  setCorner( Qt::BottomLeftCorner, Qt::LeftDockWidgetArea );
+  setCorner( Qt::BottomRightCorner, Qt::RightDockWidgetArea );
+
   ui->dockWidget_histogram->hide();
   ui->dockWidget_info->hide();
   ui->dockWidget_animation->hide();
+#ifdef USE_GAMEPAD
   ui->dockWidget_gamepad_dock->hide();
+#endif
   ui->dockWidget_queue_dock->hide();
 }
 
@@ -223,7 +237,7 @@ void RenderWindow::slotMenuSaveImageJPEG()
                                       0.0,
                                       cProgressText::progress_IMAGE);
     gApplication->processEvents();
-    SaveImage(filename, IMAGE_FILE_TYPE_JPG, gMainInterface->mainImage);
+		SaveImage(filename, ImageFileSave::IMAGE_FILE_TYPE_JPG, gMainInterface->mainImage, gMainInterface->mainWindow);
     cProgressText::ProgressStatusText(tr("Saving %1 image").arg("JPG"),
                                       tr("Saving image finished"),
                                       1.0,
@@ -248,16 +262,8 @@ void RenderWindow::slotMenuSaveImagePNG()
   {
     filenames = dialog.selectedFiles();
     QString filename = QDir::toNativeSeparators(filenames.first());
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("PNG"),
-                                      tr("Saving image started"),
-                                      0.0,
-                                      cProgressText::progress_IMAGE);
     gApplication->processEvents();
-    SaveImage(filename, IMAGE_FILE_TYPE_PNG, gMainInterface->mainImage);
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("PNG"),
-                                      tr("Saving image finished"),
-                                      1.0,
-                                      cProgressText::progress_IMAGE);
+		SaveImage(filename, ImageFileSave::IMAGE_FILE_TYPE_PNG, gMainInterface->mainImage, gMainInterface->mainWindow);
     gApplication->processEvents();
     systemData.lastImageFile = filename;
   }
@@ -279,17 +285,9 @@ void RenderWindow::slotMenuSaveImageEXR()
   {
     filenames = dialog.selectedFiles();
     QString filename = QDir::toNativeSeparators(filenames.first());
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("EXR"),
-                                      tr("Saving EXR image started"),
-                                      0.0,
-                                      cProgressText::progress_IMAGE);
-    gApplication->processEvents();
-    SaveImage(filename, IMAGE_FILE_TYPE_EXR, gMainInterface->mainImage);
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("EXR"),
-                                      tr("Saving EXR image finished"),
-                                      1.0,
-                                      cProgressText::progress_IMAGE);
-    gApplication->processEvents();
+		gApplication->processEvents();
+		SaveImage(filename, ImageFileSave::IMAGE_FILE_TYPE_EXR, gMainInterface->mainImage, gMainInterface->mainWindow);
+		gApplication->processEvents();
     systemData.lastImageFile = filename;
   }
 }
@@ -311,17 +309,9 @@ void RenderWindow::slotMenuSaveImageTIFF()
   {
     filenames = dialog.selectedFiles();
     QString filename = QDir::toNativeSeparators(filenames.first());
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("TIFF"),
-                                      tr("Saving TIFF image started"),
-                                      0.0,
-                                      cProgressText::progress_IMAGE);
-    gApplication->processEvents();
-    SaveImage(filename, IMAGE_FILE_TYPE_TIFF, gMainInterface->mainImage);
-    cProgressText::ProgressStatusText(tr("Saving %1 image").arg("TIFF"),
-                                      tr("Saving TIFF image finished"),
-                                      1.0,
-                                      cProgressText::progress_IMAGE);
-    gApplication->processEvents();
+		gApplication->processEvents();
+		SaveImage(filename, ImageFileSave::IMAGE_FILE_TYPE_TIFF, gMainInterface->mainImage, gMainInterface->mainWindow);
+		gApplication->processEvents();
     systemData.lastImageFile = filename;
   }
 }
@@ -347,8 +337,9 @@ void RenderWindow::slotMenuSaveImagePNG16()
                                       0.0,
                                       cProgressText::progress_IMAGE);
     gApplication->processEvents();
-    structSaveImageChannel saveImageChannel(IMAGE_CONTENT_COLOR, IMAGE_CHANNEL_QUALITY_16, "");
-    SavePNG(filename, gMainInterface->mainImage, saveImageChannel, false);
+		ImageFileSave::structSaveImageChannel saveImageChannel(
+			ImageFileSave::IMAGE_CONTENT_COLOR, ImageFileSave::IMAGE_CHANNEL_QUALITY_16, "");
+		ImageFileSavePNG::SavePNG(filename, gMainInterface->mainImage, saveImageChannel, false);
     cProgressText::ProgressStatusText(tr("Saving %1 image").arg("16-bit PNG"),
                                       tr("Saving image finished"),
                                       1.0,
@@ -378,8 +369,9 @@ void RenderWindow::slotMenuSaveImagePNG16Alpha()
                                       0.0,
                                       cProgressText::progress_IMAGE);
     gApplication->processEvents();
-    structSaveImageChannel saveImageChannel(IMAGE_CONTENT_COLOR, IMAGE_CHANNEL_QUALITY_16, "");
-    SavePNG(filename, gMainInterface->mainImage, saveImageChannel, true);
+		ImageFileSave::structSaveImageChannel saveImageChannel(
+			ImageFileSave::IMAGE_CONTENT_COLOR, ImageFileSave::IMAGE_CHANNEL_QUALITY_16, "");
+		ImageFileSavePNG::SavePNG(filename, gMainInterface->mainImage, saveImageChannel, true);
     cProgressText::ProgressStatusText(tr("Saving %1 image").arg("16-bit PNG + alpha channel"),
                                       tr("Saving image finished"),
                                       1.0,
@@ -392,7 +384,7 @@ void RenderWindow::slotMenuSaveImagePNG16Alpha()
 void RenderWindow::slotMenuSaveSettings()
 {
   cSettings parSettings(cSettings::formatCondensedText);
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read);
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
   parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
 
   QFileDialog dialog(this);
@@ -423,7 +415,7 @@ void RenderWindow::slotMenuSaveSettings()
 void RenderWindow::slotMenuSaveSettingsToClipboard()
 {
   cSettings parSettings(cSettings::formatCondensedText);
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read);
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
   parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
   parSettings.SaveToClipboard();
   cErrorMessage::showMessage(QObject::tr("Settings saved to clipboard"),
@@ -579,7 +571,8 @@ void RenderWindow::slotUpdateDocksandToolbarbyView()
 
 void RenderWindow::slotStackAllDocks()
 {
-  tabifyDockWidget(ui->dockWidget_effects, ui->dockWidget_image_adjustments);
+  tabifyDockWidget(ui->dockWidget_materialEditor, ui->dockWidget_effects);
+	tabifyDockWidget(ui->dockWidget_effects, ui->dockWidget_image_adjustments);
   tabifyDockWidget(ui->dockWidget_image_adjustments, ui->dockWidget_rendering_engine);
   tabifyDockWidget(ui->dockWidget_rendering_engine, ui->dockWidget_fractal);
   tabifyDockWidget(ui->dockWidget_fractal, ui->dockWidget_histogram);
