@@ -25,11 +25,17 @@
 #include "my_ui_loader.h"
 #include "initparameters.hpp"
 #include "settings.hpp"
-#include "preferencesdialog.h"
+#include "../qt/preferencesdialog.h"
+#include "../qt/voxel_export_dialog.h"
+#include "interface.hpp"
+#include "error_message.hpp"
+#include "image_scale.hpp"
+#include "../qt/thumbnail_widget.h"
+#include "synchronize_interface.hpp"
 
 void RenderWindow::slotStartRender(void)
 {
-  gMainInterface->StartRender();
+	gMainInterface->StartRender();
 }
 
 void RenderWindow::slotStopRender(void)
@@ -66,10 +72,10 @@ void RenderWindow::slotChangedComboFractal(int indexInComboBox)
       layout->addWidget(fractalWidgets[fractalNumber]);
       uiFile.close();
       fractalWidgets[fractalNumber]->show();
-      gMainInterface->ConnectSignalsForSlidersInWindow(fractalWidgets[fractalNumber]);
-      gMainInterface->SynchronizeInterfaceWindow(fractalWidgets[fractalNumber],
+      automatedWidgets->ConnectSignalsForSlidersInWindow(fractalWidgets[fractalNumber]);
+      SynchronizeInterfaceWindow(fractalWidgets[fractalNumber],
                                                  &gParFractal->at(fractalNumber),
-                                                 cInterface::write);
+                                                 qInterface::write);
 
       QFrame *frame = ui->tabWidget_fractals->findChild<QFrame*>("frame_iterations_formula_"
           + QString::number(fractalNumber + 1));
@@ -240,7 +246,6 @@ void RenderWindow::slotChangedCheckBoxHybridFractal(int state)
 
 void RenderWindow::slotChangedCheckBoxBooleanOperators(bool state)
 {
-  //TODO change number of fractals to 9 for boolean operators
   if (state) ui->checkBox_hybrid_fractal_enable->setChecked(false);
   gApplication->processEvents();
 
@@ -264,9 +269,12 @@ void RenderWindow::slotChangedCheckBoxBooleanOperators(bool state)
           + QString::number(i))->setVisible(false);
     else ui->tabWidget_fractals->findChild<QGroupBox*>("groupBox_c_constant_addition_"
         + QString::number(i))->setVisible(state);
+
+		ui->tabWidget_fractals->findChild<QGroupBox*>("groupBox_material_fractal_" + QString::number(i))->setVisible(state);
   }
 
   ui->comboBox_delta_DE_function->setEnabled(!state);
+  ui->groupBox_material_fractal->setVisible(!state);
 }
 
 void RenderWindow::slotChangedCheckBoxJuliaMode(bool state)
@@ -423,16 +431,6 @@ void RenderWindow::slotChangedComboPerspectiveType(int index)
   }
 }
 
-void RenderWindow::slotChangedSpinBoxPaletteOffset(double value)
-{
-  ui->colorpalette_surface_color_palette->SetOffset(value);
-}
-
-void RenderWindow::slotChangedSpinBoxPaletteSize(int value)
-{
-  ui->slider_coloring_palette_offset->setMaximum(value * 100);
-}
-
 void RenderWindow::slotMouseMovedOnImage(int x, int y)
 {
   (void) x;
@@ -550,7 +548,7 @@ void RenderWindow::slotChangedCheckBoxCursorVisibility(int state)
 //adds dynamic actions to the toolbar (example settings)
 void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 {
-  WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) started");
+  WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) started", 2);
   QDir toolbarDir = QDir(systemData.dataDirectory + "toolbar");
   toolbarDir.setSorting(QDir::Time);
   QStringList toolbarFiles = toolbarDir.entryList(QDir::NoDotAndDotDot | QDir::Files);
@@ -587,7 +585,7 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
 
     if (QFileInfo(filename).suffix() == QString("fract"))
     {
-      WriteLogString("Generating thumbnail for preset", filename);
+      WriteLogString("Generating thumbnail for preset", filename, 2);
       cSettings parSettings(cSettings::formatFullText);
       parSettings.BeQuiet(true);
 
@@ -598,6 +596,13 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
         InitParams(par);
         for (int i = 0; i < NUMBER_OF_FRACTALS; i++)
           InitFractalParams(&parFractal->at(i));
+
+      	/****************** TEMPORARY CODE FOR MATERIALS *******************/
+
+      	InitMaterialParams(1, par);
+
+      	/*******************************************************************/
+
         if (parSettings.Decode(par, parFractal))
         {
           thumbWidget = new cThumbnailWidget(gPar->Get<int>("toolbar_icon_size"), gPar->Get<int>("toolbar_icon_size"), 2, this);
@@ -645,13 +650,13 @@ void RenderWindow::slotPopulateToolbar(bool completeRefresh)
                         this,
                         SLOT(slotMenuRemovePreset(QString)));
 
-  WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) finished");
+  WriteLog("cInterface::PopulateToolbar(QWidget *window, QToolBar *toolBar) finished", 2);
 }
 
 void RenderWindow::slotPresetAddToToolbar()
 {
   cSettings parSettings(cSettings::formatCondensedText);
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read);
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
   parSettings.CreateText(gPar, gParFractal, gAnimFrames, gKeyframes);
   QString filename = systemData.dataDirectory + "toolbar/" + parSettings.GetHashCode() + ".fract";
   parSettings.SaveToFile(filename);
@@ -664,7 +669,8 @@ void RenderWindow::slotMenuLoadPreset(QString filename)
   parSettings.LoadFromFile(filename);
   parSettings.Decode(gPar, gParFractal);
   gMainInterface->RebuildPrimitives(gPar);
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+  gMainInterface->materialListModel->Regenerate();
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
   gMainInterface->ComboMouseClickUpdate();
   systemData.lastSettingsFile = gPar->Get<QString>("default_settings_path") + QDir::separator()
       + QFileInfo(filename).fileName();
@@ -710,7 +716,7 @@ void RenderWindow::slotUpdateProgressAndStatus(const QString &text, const QStrin
     double progress, cProgressText::enumProgressType progressType)
 {
   ui->statusbar->showMessage(text, 0);
-  QProgressBar *progressBar = NULL;
+  MyProgressBar *progressBar = NULL;
   bool isQueue = this->sender() && this->sender()->objectName() == "Queue";
   switch (progressType)
   {
@@ -741,7 +747,7 @@ void RenderWindow::slotUpdateProgressAndStatus(const QString &text, const QStrin
 
 void RenderWindow::slotUpdateProgressHide(cProgressText::enumProgressType progressType)
 {
-  QProgressBar *progressBar = NULL;
+  MyProgressBar *progressBar = NULL;
   switch (progressType)
   {
     case cProgressText::progress_IMAGE:
@@ -763,8 +769,26 @@ void RenderWindow::slotUpdateProgressHide(cProgressText::enumProgressType progre
 
 void RenderWindow::slotMenuProgramSettings()
 {
-  cPreferencesDialog *preferencesDialog = new cPreferencesDialog;
-  preferencesDialog->show();
+	if(!preferencesDialog)
+	{
+		preferencesDialog = new cPreferencesDialog;
+	}
+
+	preferencesDialog->show();
+	preferencesDialog->raise();
+	preferencesDialog->activateWindow();
+}
+
+void RenderWindow::slotExportVoxelLayers()
+{
+	if(!voxelExportDialog)
+	{
+		voxelExportDialog = new cVoxelExportDialog;
+	}
+
+	voxelExportDialog->show();
+	voxelExportDialog->raise();
+	voxelExportDialog->activateWindow();
 }
 
 void RenderWindow::slotUpdateStatistics(cStatistics stat)
@@ -793,7 +817,7 @@ void RenderWindow::slotFractalSwap(int swapA, int swapB)
   // qDebug() << "swapping " << swapA << " with " << swapB;
 
   // read all data from ui
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::read);
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::read);
 
   // swap formula specific fields in gPar
   QStringList gParFormulaSpecificFields;
@@ -811,15 +835,15 @@ void RenderWindow::slotFractalSwap(int swapA, int swapB)
   }
 
   // swap formula specific fields in gParFractal by swapping whole container
-  gMainInterface->SynchronizeInterfaceWindow(fractalWidgets[swapA],
+  SynchronizeInterfaceWindow(fractalWidgets[swapA],
                                              &gParFractal->at(swapB),
-                                             cInterface::read);
-  gMainInterface->SynchronizeInterfaceWindow(fractalWidgets[swapB],
+                                             qInterface::read);
+  SynchronizeInterfaceWindow(fractalWidgets[swapB],
                                              &gParFractal->at(swapA),
-                                             cInterface::read);
+                                             qInterface::read);
 
   // write swapped changes to ui
-  gMainInterface->SynchronizeInterface(gPar, gParFractal, cInterface::write);
+  gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
 }
 
 void RenderWindow::slotChangedCheckBoxUseDefaultBailout(int state)
@@ -828,16 +852,6 @@ void RenderWindow::slotChangedCheckBoxUseDefaultBailout(int state)
   ui->logedit_bailout->setEnabled(!state);
 }
 
-void RenderWindow::slotChangedComboFractalColoringAlgorithm(int index)
-{
-  fractal::enumFractalColoringAlgorithm selection = (fractal::enumFractalColoringAlgorithm)index;
-  ui->slider_fractal_coloring_sphere_radius->setEnabled(selection == fractal::fractalColoringSphere);
-  ui->spinbox_fractal_coloring_sphere_radius->setEnabled(selection == fractal::fractalColoringSphere);
-  ui->vect3_fractal_coloring_line_direction_x->setEnabled(selection == fractal::fractalColoringLine);
-  ui->vect3_fractal_coloring_line_direction_y->setEnabled(selection == fractal::fractalColoringLine);
-  ui->vect3_fractal_coloring_line_direction_z->setEnabled(selection == fractal::fractalColoringLine);
-}
-
 void RenderWindow::slotChangedCheckBoxDOFHDR(int state)
 {
   ui->pushButton_DOF_update->setEnabled(!state);
@@ -847,3 +861,74 @@ void RenderWindow::slotChangedCheckBoxDOFHDR(int state)
     ui->comboBox_ambient_occlusion_mode->setCurrentIndex(0);
   }
 }
+
+void RenderWindow::slotAutoRefresh(void)
+{
+	gMainInterface->PeriodicRefresh();
+}
+
+void RenderWindow::slotMaterialSelected(int matIndex)
+{
+	gMainInterface->MaterialSelected(matIndex);
+	ui->dockWidget_materialEditor->raise();
+}
+
+void RenderWindow::slotMaterialEdited()
+{
+	SynchronizeInterfaceWindow(gMainInterface->materialEditor, gPar, qInterface::write);
+}
+
+void RenderWindow::slotToggledFractalEnable(int fractalIndex, bool enabled)
+{
+	ui->tabWidget_fractals->widget(fractalIndex - 1)->setEnabled(enabled);
+}
+
+void RenderWindow::slotQualityPresetVeryLow()
+{
+	gPar->Set("DE_factor", 1.0);
+	gPar->Set("ambient_occlusion_enabled", false);
+	gPar->Set("shadows_enabled", false);
+	gPar->Set("raytraced_reflections", false);
+	gPar->Set("detail_level", 0.5);
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+}
+void RenderWindow::slotQualityPresetLow()
+{
+	gPar->Set("DE_factor", 1.0);
+	gPar->Set("ambient_occlusion_enabled", false);
+	gPar->Set("shadows_enabled", true);
+	gPar->Set("raytraced_reflections", true);
+	gPar->Set("detail_level", 1.0);
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+}
+void RenderWindow::slotQualityPresetNormal()
+{
+	gPar->Set("DE_factor", 1.0);
+	gPar->Set("ambient_occlusion_enabled", true);
+	gPar->Set("ambient_occlusion_mode", (int) params::AOmodeScreenSpace);
+	gPar->Set("ambient_occlusion_quality", 4);
+	gPar->Set("shadows_enabled", true);
+	gPar->Set("raytraced_reflections", true);
+	gPar->Set("detail_level", 1.0);
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+}
+void RenderWindow::slotQualityPresetHigh()
+{
+	gPar->Set("DE_factor", 0.2);
+	gPar->Set("ambient_occlusion_enabled", true);
+	gPar->Set("ambient_occlusion_mode", (int) params::AOmodeMultipeRays);
+	gPar->Set("ambient_occlusion_quality", 4);
+	gPar->Set("shadows_enabled", true);
+	gPar->Set("raytraced_reflections", true);
+	gPar->Set("detail_level", 1.0);
+	gMainInterface->SynchronizeInterface(gPar, gParFractal, qInterface::write);
+}
+
+void RenderWindow::slotGroupCheckJuliaModeToggled(bool state)
+{
+	if(state)
+	{
+		gMainInterface->DisablePeriodicRefresh();
+	}
+}
+
