@@ -74,8 +74,6 @@ cRenderWorker::~cRenderWorker()
 void cRenderWorker::doWork(void)
 {
 	// here will be rendering thread
-	QTextStream out(stdout);
-
 	int width = image->GetWidth();
 	int height = image->GetHeight();
 	double aspectRatio = (double) width / height;
@@ -105,7 +103,7 @@ void cRenderWorker::doWork(void)
 		//main loop for x
 		for (int xs = 0; xs < width; xs += scheduler->GetProgressiveStep())
 		{
-
+			if(systemData.globalStopRequest) break;
 			//break if by coincidence this thread started rendering the same line as some other
 			lastLineWasBroken = false;
 			if (scheduler->ShouldIBreak(threadData->id, ys))
@@ -139,10 +137,10 @@ void cRenderWorker::doWork(void)
 			//---------------- 1us -------------
 
 			//Ray marching
-			CVector3 point;
 			CVector3 startRay = start;
 			sRGBAfloat resultShader;
 			sRGBAfloat objectColour;
+			CVector3 normal;
 			double depth = 1e20;
 			double opacity = 1.0;
 
@@ -180,6 +178,7 @@ void cRenderWorker::doWork(void)
 				depth = recursionOut.rayMarchingOut.depth;
 				if (!recursionOut.found) depth = 1e20;
 				opacity = recursionOut.fogOpacity;
+				normal = recursionOut.normal;
 			}
 
 			sRGBfloat pixel2;
@@ -194,6 +193,15 @@ void cRenderWorker::doWork(void)
 			colour.G = objectColour.G * 255;
 			colour.B = objectColour.B * 255;
 
+			sRGBfloat normalFloat;
+			if(image->GetImageOptional()->optionalNormal)
+			{
+				CVector3 normalRotated = mRotInv.RotateVector(normal);
+				normalFloat.R = (1.0 + normalRotated.x) / 2.0;
+				normalFloat.G = (1.0 + normalRotated.z) / 2.0;
+				normalFloat.B = 1.0 - normalRotated.y;
+			}
+
 			for (int xx = 0; xx < scheduler->GetProgressiveStep(); ++xx)
 			{
 				for (int yy = 0; yy < scheduler->GetProgressiveStep(); ++yy)
@@ -203,6 +211,8 @@ void cRenderWorker::doWork(void)
 					image->PutPixelAlpha(screenPoint.x + xx, screenPoint.y + yy, alpha);
 					image->PutPixelZBuffer(screenPoint.x + xx, screenPoint.y + yy, (float) depth);
 					image->PutPixelOpacity(screenPoint.x + xx, screenPoint.y + yy, opacity16);
+					if(image->GetImageOptional()->optionalNormal)
+						image->PutPixelNormal(screenPoint.x + xx, screenPoint.y + yy, normalFloat);
 				}
 			}
 
@@ -251,6 +261,8 @@ void cRenderWorker::PrepareMainVectors(void)
 
 	mRot.RotateZ(-params->sweetSpotHAngle);
 	mRot.RotateX(params->sweetSpotVAngle);
+
+	mRotInv = mRot.Transpose();
 }
 
 //reflection data
@@ -318,7 +330,7 @@ void cRenderWorker::PrepareAOVectors(void)
 }
 
 //calculation of distance where ray-marching stops
-double cRenderWorker::CalcDistThresh(CVector3 point)
+double cRenderWorker::CalcDistThresh(CVector3 point) const
 {
 	double distThresh;
 	if (params->constantDEThreshold) distThresh = params->DEThresh;
@@ -332,7 +344,7 @@ double cRenderWorker::CalcDistThresh(CVector3 point)
 }
 
 //calculation of "voxel" size
-double cRenderWorker::CalcDelta(CVector3 point)
+double cRenderWorker::CalcDelta(CVector3 point) const
 {
 	double delta;
 	delta = (params->camera - point).Length() * params->resolution * params->fov;
@@ -355,38 +367,46 @@ CVector3 cRenderWorker::RayMarching(sRayMarchingIn &in, sRayMarchingInOut *inOut
 	int counter = 0;
 	double step = 0.0;
 	(*inOut->buffCount) = 0;
-	double distThresh;
+	double distThresh = 0;
+	out->objectId = 0;
 
 	//qDebug() << "Start ************************";
+
+	CVector3 lastPoint, lastGoodPoint;
+	bool deadComputationFound = false;
+
 	for (int i = 0; i < 10000; i++)
 	{
+		lastGoodPoint = lastPoint;
+		lastPoint = point;
+
 		counter++;
 
 		point = in.start + in.direction * scan;
 
+		if (point == lastPoint || point == point/0.0) //detection of dead calculation
+		{
+			//qWarning() << "Dead computation\n"
+			//		<< "Point:" << point.Debug()
+			//		<< "\nPrevious point:" << lastPoint.Debug();
+			point = lastPoint;
+			found = true;
+			deadComputationFound = true;
+			break;
+		}
+
 		distThresh = CalcDistThresh(point);
 
 		sDistanceIn distanceIn(point, distThresh, false);
 		sDistanceOut distanceOut;
-		dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+		dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
 		//qDebug() <<"thresh" <<  distThresh << "dist" << dist << "scan" << scan;
 		if (in.invertMode)
 		{
 			dist = distThresh * 1.99 - dist;
 			if (dist < 0.0) dist = 0.0;
 		}
-		out->object = distanceOut.object;
-		out->formulaIndex = distanceOut.formulaIndex;
-		if (out->object == fractal::objFractal)
-		{
-			out->objectReflect = params->reflect;
-			out->objectColor = sRGB(0, 0, 0);
-		}
-		else
-		{
-			out->objectReflect = distanceOut.objectReflect;
-			out->objectColor = distanceOut.objectColor;
-		}
+		out->objectId = distanceOut.objectId;
 
 		//-------------------- 4.18us for Calculate distance --------------
 
@@ -425,11 +445,12 @@ CVector3 cRenderWorker::RayMarching(sRayMarchingIn &in, sRayMarchingInOut *inOut
 		{
 			break;
 		}
+
 	}
 	//------------- 83.2473 us for RayMarching loop -------------------------
 
 	//qDebug() << "------------ binary search";
-	if (found && in.binaryEnable)
+	if (found && in.binaryEnable && !deadComputationFound)
 	{
 		step *= 0.5;
 		for (int i = 0; i < 30; i++)
@@ -457,7 +478,7 @@ CVector3 cRenderWorker::RayMarching(sRayMarchingIn &in, sRayMarchingInOut *inOut
 
 			sDistanceIn distanceIn(point, distThresh, false);
 			sDistanceOut distanceOut;
-			dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+			dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
 
 			//qDebug() << "i" << i <<"thresh" <<  distThresh << "dist" << dist << "scan" << scan << "step" << step;
 
@@ -467,19 +488,7 @@ CVector3 cRenderWorker::RayMarching(sRayMarchingIn &in, sRayMarchingInOut *inOut
 				if (dist < 0.0) dist = 0.0;
 			}
 
-			out->object = distanceOut.object;
-			out->formulaIndex = distanceOut.formulaIndex;
-
-			if (out->object == fractal::objFractal)
-			{
-				out->objectReflect = params->reflect;
-				out->objectColor = sRGB(0, 0, 0);
-			}
-			else
-			{
-				out->objectReflect = distanceOut.objectReflect;
-				out->objectColor = distanceOut.objectColor;
-			}
+			out->objectId = distanceOut.objectId;
 
 			data->statistics.histogramIterations.Add(distanceOut.iters);
 			data->statistics.totalNumberOfIterations += distanceOut.totalIters;
@@ -539,39 +548,65 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 	shaderInputData.depth = rayMarchingOut.depth;
 	shaderInputData.stepCount = *inOut.rayMarchingInOut.buffCount;
 	shaderInputData.stepBuff = inOut.rayMarchingInOut.stepBuff;
-	shaderInputData.objectType = rayMarchingOut.object;
-	shaderInputData.objectColor = rayMarchingOut.objectColor;
-	shaderInputData.formulaIndex = rayMarchingOut.formulaIndex;
 	shaderInputData.invertMode = in.calcInside;
+	shaderInputData.objectId = rayMarchingOut.objectId;
+
+	cObjectData objectData = data->objectData[shaderInputData.objectId];
+	shaderInputData.material = &data->materials[objectData.materialId];
 
 	sRGBAfloat reflectShader = in.resultShader;
-	double reflect = rayMarchingOut.objectReflect;
+	double reflect = shaderInputData.material->reflectance;
+
 	sRGBAfloat transparentShader = in.resultShader;
-	double transparent = params->transparencyOfSurface;
-	sRGBfloat transparentColor = sRGBfloat(params->transparencyInteriorColor.R / 65536.0,
-																				 params->transparencyInteriorColor.G / 65536.0,
-																				 params->transparencyInteriorColor.B / 65536.0);
+	double transparent = shaderInputData.material->transparencyOfSurface;
+	sRGBfloat transparentColor = sRGBfloat(shaderInputData.material->transparencyInteriorColor.R / 65536.0,
+																				 shaderInputData.material->transparencyInteriorColor.G / 65536.0,
+																				 shaderInputData.material->transparencyInteriorColor.B / 65536.0);
 	resultShader.R = transparentColor.R;
 	resultShader.G = transparentColor.G;
 	resultShader.B = transparentColor.B;
 
+	CVector3 vn;
+
 	//if found any object
 	if (rayMarchingOut.found)
 	{
 		//calculate normal vector
-		CVector3 vn = CalculateNormals(shaderInputData);
+		vn = CalculateNormals(shaderInputData);
+		shaderInputData.normal = vn;
+
+		//letting colors from textures (before normal map shader)
+		if(shaderInputData.material->colorTexture.IsLoaded())
+			shaderInputData.texColor = TextureShader(shaderInputData, cMaterial::texColor, shaderInputData.material);
+		else
+			shaderInputData.texColor = sRGBfloat(1.0, 1.0, 1.0);
+
+		if(shaderInputData.material->luminosityTexture.IsLoaded())
+			shaderInputData.texLuminosity = TextureShader(shaderInputData, cMaterial::texLuminosity, shaderInputData.material);
+		else
+			shaderInputData.texLuminosity = sRGBfloat(0.0, 0.0, 0.0);
+
+		if(shaderInputData.material->diffusionTexture.IsLoaded())
+			shaderInputData.texDiffuse = TextureShader(shaderInputData, cMaterial::texDiffuse, shaderInputData.material);
+		else
+			shaderInputData.texDiffuse = sRGBfloat(1.0, 1.0, 1.0);
+
+		if(shaderInputData.material->normalMapTexture.IsLoaded())
+		{
+			vn = NormalMapShader(shaderInputData);
+		}
 
 		//prepare refraction values
 		double n1, n2;
-		if (in.calcInside) //if trance is inside the object
+		if (in.calcInside) //if trace is inside the object
 		{
-			n1 = params->transparencyIndexOfRefraction; //reverse refractive indices
+			n1 = shaderInputData.material ->transparencyIndexOfRefraction; //reverse refractive indices
 			n2 = 1.0;
 		}
 		else
 		{
 			n1 = 1.0;
-			n2 = params->transparencyIndexOfRefraction;
+			n2 = shaderInputData.material ->transparencyIndexOfRefraction;
 			;
 		}
 
@@ -670,7 +705,7 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 		double reflectance = 1.0;
 		double reflectanceN = 1.0;
 
-		if (params->fresnelReflectance)
+		if (shaderInputData.material->fresnelReflectance)
 		{
 			reflectance = Reflectance(vn, in.rayMarchingIn.direction, n1, n2);
 			if (reflectance < 0.0) reflectance = 0.0;
@@ -685,6 +720,13 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 
 		if (reflectionsMax > 0)
 		{
+			sRGBfloat reflectDiffused;
+			double diffIntes = shaderInputData.material->diffussionTextureIntensity;
+			double diffIntesN = 1.0 - diffIntes;
+			reflectDiffused.R = reflect * shaderInputData.texDiffuse.R * diffIntes + reflect * diffIntesN;
+			reflectDiffused.G = reflect * shaderInputData.texDiffuse.G * diffIntes + reflect * diffIntesN;
+			reflectDiffused.B = reflect * shaderInputData.texDiffuse.B * diffIntes + reflect * diffIntesN;
+
 			resultShader.R = transparentShader.R * transparent * reflectanceN
 					+ (1.0 - transparent * reflectanceN) * resultShader.R;
 			resultShader.G = transparentShader.G * transparent * reflectanceN
@@ -692,12 +734,12 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 			resultShader.B = transparentShader.B * transparent * reflectanceN
 					+ (1.0 - transparent * reflectanceN) * resultShader.B;
 
-			resultShader.R = reflectShader.R * reflect * reflectance
-					+ (1.0 - reflect * reflectance) * resultShader.R;
-			resultShader.G = reflectShader.G * reflect * reflectance
-					+ (1.0 - reflect * reflectance) * resultShader.G;
-			resultShader.B = reflectShader.B * reflect * reflectance
-					+ (1.0 - reflect * reflectance) * resultShader.B;
+			resultShader.R = reflectShader.R * reflectDiffused.R * reflectance
+					+ (1.0 - reflectDiffused.R * reflectance) * resultShader.R;
+			resultShader.G = reflectShader.G * reflectDiffused.G * reflectance
+					+ (1.0 - reflectDiffused.G * reflectance) * resultShader.G;
+			resultShader.B = reflectShader.B * reflectDiffused.B * reflectance
+					+ (1.0 - reflectDiffused.B * reflectance) * resultShader.B;
 		}
 		if(resultShader.R < 0.0) resultShader.R = 0.0;
 		if(resultShader.G < 0.0) resultShader.G = 0.0;
@@ -708,6 +750,7 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 		backgroundShader = BackgroundShader(shaderInputData);
 		resultShader = backgroundShader;
 		shaderInputData.depth = 1e20;
+		vn =  mRot.RotateVector(CVector3(0.0, -1.0, 0.0));
 	}
 
 	sRGBAfloat opacityOut;
@@ -725,7 +768,7 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 			//transparentColor.G = color.G;
 			//transparentColor.B = color.B;
 
-			double opacity = -log(params->transparencyOfInterior) * step;
+			double opacity = (-1.0 + 1.0/shaderInputData.material ->transparencyOfInterior) * step;
 			if (opacity > 1.0) opacity = 1.0;
 
 			resultShader.R = opacity * transparentColor.R + (1.0 - opacity) * resultShader.R;
@@ -747,6 +790,7 @@ cRenderWorker::sRayRecursionOut cRenderWorker::RayRecursion(sRayRecursionIn in,
 	out.resultShader = resultShader;
 	out.found = (shaderInputData.depth == 1e20) ? false : true;
 	out.fogOpacity = opacityOut.R;
+	out.normal = vn;
 
 	return out;
 }
