@@ -22,6 +22,9 @@
 
 #include "render_worker.hpp"
 #include "calculate_distance.hpp"
+#include "compute_fractal.hpp"
+#include "common_math.h"
+#include "texture_mapping.hpp"
 
 sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloat *surfaceColour,
 		sRGBAfloat *specularOut)
@@ -31,8 +34,10 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 	//normal vector
 	CVector3 vn = _input.normal;
 	sShaderInputData input = _input;
+	cMaterial *mat = input.material;
 	input.normal = vn;
 
+	//main light
 	sRGBAfloat mainLight;
 	mainLight.R = params->mainLightIntensity * params->mainLightColour.R / 65536.0;
 	mainLight.G = params->mainLightIntensity * params->mainLightColour.G / 65536.0;
@@ -43,9 +48,9 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 	if (params->mainLightEnable)
 	{
 		shade = MainShading(input);
-		shade.R = params->mainLightIntensity * ((1.0 - params->shading) + params->shading * shade.R);
-		shade.G = params->mainLightIntensity * ((1.0 - params->shading) + params->shading * shade.G);
-		shade.B = params->mainLightIntensity * ((1.0 - params->shading) + params->shading * shade.B);
+		shade.R = params->mainLightIntensity * ((1.0 - mat->shading) + mat->shading * shade.R);
+		shade.G = params->mainLightIntensity * ((1.0 - mat->shading) + mat->shading * shade.G);
+		shade.B = params->mainLightIntensity * ((1.0 - mat->shading) + mat->shading * shade.B);
 	}
 
 	//calculate shadow
@@ -57,13 +62,18 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 	if (params->mainLightEnable)
 	{
 		specular = MainSpecular(input);
-		specular.R *= params->specular;
-		specular.G *= params->specular;
-		specular.B *= params->specular;
+		specular.R *= mat->specular;
+		specular.G *= mat->specular;
+		specular.B *= mat->specular;
 	}
 
 	//calculate surface colour
 	sRGBAfloat colour = SurfaceColour(input);
+	double texColInt = mat->colorTextureIntensity;
+	double texColIntN = 1.0 - mat->colorTextureIntensity;
+	colour.R *= input.texColor.R * texColInt + texColIntN;
+	colour.G *= input.texColor.G * texColInt + texColIntN;
+	colour.B *= input.texColor.B * texColInt + texColIntN;
 	*surfaceColour = colour;
 
 	//ambient occlusion
@@ -91,9 +101,9 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 	{
 		envMapping = EnvMapping(input);
 	}
-	envMapping.R *= params->reflect;
-	envMapping.G *= params->reflect;
-	envMapping.B *= params->reflect;
+	envMapping.R *= mat->reflectance * input.texDiffuse.R;
+	envMapping.G *= mat->reflectance * input.texDiffuse.G;
+	envMapping.B *= mat->reflectance * input.texDiffuse.B;
 
 	//additional lights
 	sRGBAfloat auxLights;
@@ -108,6 +118,12 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 		fakeLights = FakeLights(input, &fakeLightsSpecular);
 	}
 
+	//luminosity
+	sRGBAfloat luminosity;
+	luminosity.R = input.texLuminosity.R * mat->luminosityTextureIntensity + mat->luminosity * mat->luminosityColor.R / 65536.0;
+	luminosity.G = input.texLuminosity.G * mat->luminosityTextureIntensity + mat->luminosity * mat->luminosityColor.G / 65536.0;
+	luminosity.B = input.texLuminosity.B * mat->luminosityTextureIntensity + mat->luminosity * mat->luminosityColor.B / 65536.0;
+
 	//total shader
 	output.R = envMapping.R + (ambient2.R + mainLight.R * shade.R * shadow.R) * colour.R;
 	output.G = envMapping.G + (ambient2.G + mainLight.G * shade.G * shadow.G) * colour.G;
@@ -116,6 +132,11 @@ sRGBAfloat cRenderWorker::ObjectShader(const sShaderInputData &_input, sRGBAfloa
 	output.R += (auxLights.R + fakeLights.R) * colour.R;
 	output.G += (auxLights.G + fakeLights.G) * colour.G;
 	output.B += (auxLights.B + fakeLights.B) * colour.B;
+
+	output.R += luminosity.R;
+	output.G += luminosity.G;
+	output.B += luminosity.B;
+
 	output.A = 1.0;
 
 	(*specularOut).R = auxLightsSpecular.R + fakeLightsSpecular.R
@@ -135,41 +156,61 @@ sRGBAfloat cRenderWorker::BackgroundShader(const sShaderInputData &input)
 
 	if (params->texturedBackground)
 	{
-		if (params->texturedBackgroundMapType == params::mapDoubleHemisphere)
+		switch(params->texturedBackgroundMapType)
 		{
-			double alphaTexture = input.viewVector.GetAlpha();
-			double betaTexture = input.viewVector.GetBeta();
-			int texWidth = data->textures.backgroundTexture.Width() * 0.5;
-			int texHeight = data->textures.backgroundTexture.Height();
-			int offset = 0;
-
-			if (betaTexture < 0)
+			case params::mapDoubleHemisphere:
 			{
-				betaTexture = -betaTexture;
-				alphaTexture = M_PI - alphaTexture;
-				offset = texWidth;
+				double alphaTexture = input.viewVector.GetAlpha();
+				double betaTexture = input.viewVector.GetBeta();
+				int texWidth = data->textures.backgroundTexture.Width() * 0.5;
+				int texHeight = data->textures.backgroundTexture.Height();
+				int offset = 0;
+
+				if (betaTexture < 0)
+				{
+					betaTexture = -betaTexture;
+					alphaTexture = M_PI - alphaTexture;
+					offset = texWidth;
+				}
+				double texX = 0.5 * texWidth
+						+ cos(alphaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texWidth * 0.5 + offset;
+				double texY = 0.5 * texHeight
+						+ sin(alphaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texHeight * 0.5;
+				sRGBfloat pixel = data->textures.backgroundTexture.Pixel(texX, texY);
+				pixel2.R = pixel.R;
+				pixel2.G = pixel.G;
+				pixel2.B = pixel.B;
+				break;
+			}
+			case params::mapEquirectangular:
+			{
+				double alphaTexture = fmod(-input.viewVector.GetAlpha() + 3.5 * M_PI, 2 * M_PI);
+				double betaTexture = -input.viewVector.GetBeta();
+				if (betaTexture > 0.5 * M_PI) betaTexture = 0.5 * M_PI - betaTexture;
+				if (betaTexture < -0.5 * M_PI) betaTexture = -0.5 * M_PI + betaTexture;
+				double texX = alphaTexture / (2.0 * M_PI) * data->textures.backgroundTexture.Width();
+				double texY = (betaTexture / (M_PI) + 0.5) * data->textures.backgroundTexture.Height();
+				sRGBfloat pixel = data->textures.backgroundTexture.Pixel(texX, texY);
+				pixel2.R = pixel.R;
+				pixel2.G = pixel.G;
+				pixel2.B = pixel.B;
+				break;
+			}
+			case params::mapFlat:
+			{
+				CVector3 vect = mRotInv.RotateVector(input.viewVector);
+
+				double texX = vect.x / vect.y / params->fov * params->imageHeight / params->imageWidth;
+				double texY = -vect.z / vect.y / params->fov;
+				texX  = (texX + 0.5);
+				texY  = (texY + 0.5);
+
+				sRGBfloat pixel = data->textures.backgroundTexture.Pixel(CVector2<double>(texX, texY));
+				pixel2.R = pixel.R;
+				pixel2.G = pixel.G;
+				pixel2.B = pixel.B;
+				break;
 			}
-			double texX = 0.5 * texWidth
-					+ cos(alphaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texWidth * 0.5 + offset;
-			double texY = 0.5 * texHeight
-					+ sin(alphaTexture) * (1.0 - betaTexture / (0.5 * M_PI)) * texHeight * 0.5;
-			sRGB8 pixel = data->textures.backgroundTexture.Pixel(texX, texY);
-			pixel2.R = pixel.R / 256.0;
-			pixel2.G = pixel.G / 256.0;
-			pixel2.B = pixel.B / 256.0;
-		}
-		else
-		{
-			double alphaTexture = fmod(input.viewVector.GetAlpha() + 2.5 * M_PI, 2 * M_PI);
-			double betaTexture = input.viewVector.GetBeta();
-			if (betaTexture > 0.5 * M_PI) betaTexture = 0.5 * M_PI - betaTexture;
-			if (betaTexture < -0.5 * M_PI) betaTexture = -0.5 * M_PI + betaTexture;
-			double texX = alphaTexture / (2.0 * M_PI) * data->textures.backgroundTexture.Width();
-			double texY = (betaTexture / (M_PI) + 0.5) * data->textures.backgroundTexture.Height();
-			sRGB8 pixel = data->textures.backgroundTexture.Pixel(texX, texY);
-			pixel2.R = pixel.R / 256.0;
-			pixel2.G = pixel.G / 256.0;
-			pixel2.B = pixel.B / 256.0;
 		}
 		pixel2.R *= params->background_brightness;
 		pixel2.G *= params->background_brightness;
@@ -282,21 +323,23 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 		}
 		//qDebug() << "step" << step;
 		//------------------ visible light
-		if (params->auxLightVisibility > 0)
+		if (data->lights.IsAnyLightEnabled() && params->auxLightVisibility > 0)
 		{
 			double miniStep = 0.0;
+			double lastMiniSteps = -1.0;
+
 			for (double miniSteps = 0.0; miniSteps < step; miniSteps += miniStep)
 			{
 				double lowestLightSize = 1e10;
 				double lowestLightDist = 1e10;
 				for (int i = 0; i < numberOfLights; ++i)
 				{
-					cLights::sLight light = data->lights.GetLight(i);
-					if (light.enabled)
+					const cLights::sLight* light = data->lights.GetLight(i);
+					if (light->enabled)
 					{
-						CVector3 lightDistVect = (point - input.viewVector * miniSteps) - light.position;
+						CVector3 lightDistVect = (point - input.viewVector * miniSteps) - light->position;
 						double lightDist = lightDistVect.Length();
-						double lightSize = sqrt(light.intensity) * params->auxLightVisibilitySize;
+						double lightSize = sqrt(light->intensity) * params->auxLightVisibilitySize;
 						double distToLightSurface = lightDist - lightSize;
 						if (distToLightSurface < 0.0) distToLightSurface = 0.0;
 						if (distToLightSurface <= lowestLightDist)
@@ -316,22 +359,29 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 
 				for (int i = 0; i < numberOfLights; ++i)
 				{
-					cLights::sLight light = data->lights.GetLight(i);
-					if (light.enabled)
+					const cLights::sLight *light = data->lights.GetLight(i);
+					if (light->enabled && light->intensity > 0)
 					{
-						CVector3 lightDistVect = (point - input.viewVector * miniSteps) - light.position;
+						CVector3 lightDistVect = (point - input.viewVector * miniSteps) - light->position;
 						double lightDist = lightDistVect.Length();
-						double lightSize = sqrt(light.intensity) * params->auxLightVisibilitySize;
+						double lightSize = sqrt(light->intensity) * params->auxLightVisibilitySize;
 						double r2 = lightDist / lightSize;
 						double bellFunction = 1.0 / (1.0 + pow(r2, 4.0));
 						double lightDensity = miniStep * bellFunction * params->auxLightVisibility / lightSize;
 
-						output.R += lightDensity * light.colour.R / 65536.0;
-						output.G += lightDensity * light.colour.G / 65536.0;
-						output.B += lightDensity * light.colour.B / 65536.0;
+						output.R += lightDensity * light->colour.R / 65536.0;
+						output.G += lightDensity * light->colour.G / 65536.0;
+						output.B += lightDensity * light->colour.B / 65536.0;
 						output.A += lightDensity;
 					}
 				}
+				if(miniSteps == lastMiniSteps)
+				{
+					//qWarning() << "Dead computation\n"
+					//		<< "\npoint:" << (point - input.viewVector * miniSteps).Debug();
+					break;
+				}
+				lastMiniSteps = miniSteps;
 			}
 		}
 
@@ -369,24 +419,23 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 				output.A += (shadowOutputTemp.R + shadowOutputTemp.G + shadowOutputTemp.B) / 3.0 * step
 						* params->volumetricLightIntensity[0];
 			}
-			if (i > 0)
+			if (data->lights.IsAnyLightEnabled() && i > 0)
 			{
-				cLights::sLight light = data->lights.GetLight(i - 1);
-				if (light.enabled && params->volumetricLightEnabled[i])
+				const cLights::sLight* light = data->lights.GetLight(i - 1);
+				if (light->enabled && params->volumetricLightEnabled[i])
 				{
-					CVector3 d = light.position - point;
-					double distance = d.Length();
-					double distance2 = distance * distance;
-					CVector3 lightVectorTemp = d;
+					CVector3 lightVectorTemp = light->position - point;
+					double distanceLight = lightVectorTemp.Length();
+					double distanceLight2 = distanceLight * distanceLight;
 					lightVectorTemp.Normalize();
-					double lightShadow = AuxShadow(input2, distance, lightVectorTemp);
-					output.R += lightShadow * light.colour.R / 65536.0 * params->volumetricLightIntensity[i]
-							* step / distance2;
-					output.G += lightShadow * light.colour.G / 65536.0 * params->volumetricLightIntensity[i]
-							* step / distance2;
-					output.B += lightShadow * light.colour.B / 65536.0 * params->volumetricLightIntensity[i]
-							* step / distance2;
-					output.A += lightShadow * params->volumetricLightIntensity[i] * step / distance2;
+					double lightShadow = AuxShadow(input2, distanceLight, lightVectorTemp);
+					output.R += lightShadow * light->colour.R / 65536.0 * params->volumetricLightIntensity[i]
+							* step / distanceLight2;
+					output.G += lightShadow * light->colour.G / 65536.0 * params->volumetricLightIntensity[i]
+							* step / distanceLight2;
+					output.B += lightShadow * light->colour.B / 65536.0 * params->volumetricLightIntensity[i]
+							* step / distanceLight2;
+					output.A += lightShadow * params->volumetricLightIntensity[i] * step / distanceLight2;
 				}
 			}
 		}
@@ -448,7 +497,7 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 			if (opacity > 0)
 			{
 				//fog colour
-				double iterFactor1 = (double) (L - params->iterFogOpacityTrim)
+				double iterFactor1 = (L - params->iterFogOpacityTrim)
 						/ (params->iterFogColor1Maxiter - params->iterFogOpacityTrim);
 				double k = iterFactor1;
 				if (k > 1.0) k = 1.0;
@@ -458,7 +507,7 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 				double fogColG = (params->iterFogColour1.G * kn + params->iterFogColour2.G * k);
 				double fogColB = (params->iterFogColour1.B * kn + params->iterFogColour2.B * k);
 
-				double iterFactor2 = (double) (L - params->iterFogColor1Maxiter)
+				double iterFactor2 = (L - params->iterFogColor1Maxiter)
 						/ (params->iterFogColor2Maxiter - params->iterFogColor1Maxiter);
 				double k2 = iterFactor2;
 				if (k2 < 0.0) k2 = 0.0;
@@ -487,19 +536,18 @@ sRGBAfloat cRenderWorker::VolumetricShader(const sShaderInputData &input, sRGBAf
 
 					if (i > 0)
 					{
-						cLights::sLight light = data->lights.GetLight(i - 1);
-						if (light.enabled)
+						const cLights::sLight* light = data->lights.GetLight(i - 1);
+						if (light->enabled)
 						{
-							CVector3 d = light.position - point;
-							double distance = d.Length();
-							double distance2 = distance * distance;
-							CVector3 lightVectorTemp = d;
+							CVector3 lightVectorTemp = light->position - point;
+							double distanceLight = lightVectorTemp.Length();
+							double distanceLight2 = distanceLight * distanceLight;
 							lightVectorTemp.Normalize();
-							double lightShadow = AuxShadow(input2, distance, lightVectorTemp);
-							double intensity = light.intensity * 100.0;
-							newColour.R += lightShadow * light.colour.R / 65536.0 / distance2 * intensity;
-							newColour.G += lightShadow * light.colour.G / 65536.0 / distance2 * intensity;
-							newColour.B += lightShadow * light.colour.B / 65536.0 / distance2 * intensity;
+							double lightShadow = AuxShadow(input2, distanceLight, lightVectorTemp);
+							double intensity = light->intensity * 100.0;
+							newColour.R += lightShadow * light->colour.R / 65536.0 / distanceLight2 * intensity;
+							newColour.G += lightShadow * light->colour.G / 65536.0 / distanceLight2 * intensity;
+							newColour.B += lightShadow * light->colour.B / 65536.0 / distanceLight2 * intensity;
 						}
 					}
 
@@ -575,7 +623,7 @@ sRGBAfloat cRenderWorker::MainShadow(const sShaderInputData &input)
 
 		sDistanceOut distanceOut;
 		sDistanceIn distanceIn(point2, dist_thresh, false);
-		dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+		dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
 		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 
 		if (bSoft)
@@ -633,6 +681,7 @@ sRGBAfloat cRenderWorker::FastAmbientOcclusion(const sShaderInputData &input)
 	double delta = input.distThresh;
 	double aoTemp = 0;
 	double quality = params->ambientOcclusionQuality;
+	double lastDist = 1e20;
 	for (int i = 1; i < quality * quality; i++)
 	{
 		double scan = i * i * delta;
@@ -640,7 +689,9 @@ sRGBAfloat cRenderWorker::FastAmbientOcclusion(const sShaderInputData &input)
 
 		sDistanceOut distanceOut;
 		sDistanceIn distanceIn(pointTemp, input.distThresh, false);
-		double dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+		double dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
+		if(dist > lastDist * 2) dist = lastDist * 2.0;
+		lastDist = dist;
 		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 		aoTemp += 1.0 / pow(2.0, i) * (scan - params->ambientOcclusionFastTune * dist)
 				/ input.distThresh;
@@ -674,7 +725,7 @@ sRGBAfloat cRenderWorker::AmbientOcclusion(const sShaderInputData &input)
 
 			sDistanceOut distanceOut;
 			sDistanceIn distanceIn(point2, input.distThresh, false);
-			dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+			dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
 			data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 
 			if (params->iterFogEnabled)
@@ -729,31 +780,37 @@ CVector3 cRenderWorker::CalculateNormals(const sShaderInputData &input)
 		double delta = input.delta * params->smoothness;
 		if (params->interiorMode) delta = input.distThresh * 0.2 * params->smoothness;
 
-		double s1, s2, s3, s4;
+		double sx1, sx2, sy1, sy2, sz1, sz2;
 		sDistanceOut distanceOut;
 
-		sDistanceIn distanceIn1(input.point, input.distThresh, true);
-		s1 = CalculateDistance(*params, *fractal, distanceIn1, &distanceOut);
-		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
-
 		CVector3 deltax(delta, 0.0, 0.0);
-		sDistanceIn distanceIn2(input.point + deltax, input.distThresh, true);
-		s2 = CalculateDistance(*params, *fractal, distanceIn2, &distanceOut);
+		sDistanceIn distanceIn1(input.point + deltax, input.distThresh, true);
+		sx1 = CalculateDistance(*params, *fractal, distanceIn1, &distanceOut, data);
+		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
+		sDistanceIn distanceIn2(input.point - deltax, input.distThresh, true);
+		sx2 = CalculateDistance(*params, *fractal, distanceIn2, &distanceOut, data);
 		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 
 		CVector3 deltay(0.0, delta, 0.0);
 		sDistanceIn distanceIn3(input.point + deltay, input.distThresh, true);
-		s3 = CalculateDistance(*params, *fractal, distanceIn3, &distanceOut);
+		sy1 = CalculateDistance(*params, *fractal, distanceIn3, &distanceOut, data);
+		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
+		sDistanceIn distanceIn4(input.point - deltay, input.distThresh, true);
+		sy2 = CalculateDistance(*params, *fractal, distanceIn4, &distanceOut, data);
 		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 
 		CVector3 deltaz(0.0, 0.0, delta);
-		sDistanceIn distanceIn4(input.point + deltaz, input.distThresh, true);
-		s4 = CalculateDistance(*params, *fractal, distanceIn4, &distanceOut);
+		sDistanceIn distanceIn5(input.point + deltaz, input.distThresh, true);
+		sz1 = CalculateDistance(*params, *fractal, distanceIn5, &distanceOut, data);
+		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
+		sDistanceIn distanceIn6(input.point - deltaz, input.distThresh, true);
+		sz2 = CalculateDistance(*params, *fractal, distanceIn6, &distanceOut, data);
 		data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 
-		normal.x = s2 - s1;
-		normal.y = s3 - s1;
-		normal.z = s4 - s1;
+
+		normal.x = sx1 - sx2;
+		normal.y = sy1 - sy2;
+		normal.z = sz1 - sz2;
 	}
 
 	//calculating normal vector based on average value of binary central difference
@@ -774,22 +831,29 @@ CVector3 cRenderWorker::CalculateNormals(const sShaderInputData &input)
 					point3 = input.point + point2 * delta;
 
 					sDistanceIn distanceIn(point3, input.distThresh, true);
-					double dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut);
+					double dist = CalculateDistance(*params, *fractal, distanceIn, &distanceOut, data);
 					data->statistics.totalNumberOfIterations += distanceOut.totalIters;
 					normal += (point2 * dist);
 				}
 			}
 		}
 	}
-	if (normal.x == 0 && normal.y == 0 && normal.z == 0)
+
+	if ((normal.x == 0 && normal.y == 0 && normal.z == 0))
 	{
-		normal.x = 1;
+		normal = CVector3(1.0, 0.0, 0.0);
+
 	}
 	else
 	{
 		normal.Normalize();
 	}
 
+	if(normal.IsNotANumber())
+	{
+		normal = CVector3(1.0, 0.0, 0.0);
+	}
+
 	if (input.invertMode) normal *= (-1.0);
 
 	//qDebug() << input.point.Debug() << normal.Debug();
@@ -815,11 +879,12 @@ sRGBAfloat cRenderWorker::MainSpecular(const sShaderInputData &input)
 	half.Normalize();
 	double shade2 = input.normal.Dot(half);
 	if (shade2 < 0.0) shade2 = 0.0;
-	shade2 = pow(shade2, 30.0) * 1.0;
+	double diffuse = 10.0	* (1.1 - input.material->diffussionTextureIntensity	* (input.texDiffuse.R + input.texDiffuse.G + input.texDiffuse.B) / 3.0);
+	shade2 = pow(shade2, 30.0/input.material->specularWidth / diffuse) / diffuse;
 	if (shade2 > 15.0) shade2 = 15.0;
-	specular.R = shade2;
-	specular.G = shade2;
-	specular.B = shade2;
+	specular.R = shade2 * input.material->specularColor.R / 65536.0 * (input.texDiffuse.R * 0.5 + 0.5);
+	specular.G = shade2 * input.material->specularColor.G / 65536.0 * (input.texDiffuse.G * 0.5 + 0.5);
+	specular.B = shade2 * input.material->specularColor.B / 65536.0 * (input.texDiffuse.B * 0.5 + 0.5);
 	return specular;
 }
 
@@ -830,8 +895,8 @@ sRGBAfloat cRenderWorker::EnvMapping(const sShaderInputData &input)
 	double dot = -input.viewVector.Dot(input.normal);
 	reflect = input.normal * 2.0 * dot + input.viewVector;
 
-	double alphaTexture = reflect.GetAlpha() + M_PI;
-	double betaTexture = reflect.GetBeta();
+	double alphaTexture = -reflect.GetAlpha() + M_PI;
+	double betaTexture = -reflect.GetBeta();
 	double texWidth = data->textures.envmapTexture.Width();
 	double texHeight = data->textures.envmapTexture.Height();
 
@@ -845,9 +910,20 @@ sRGBAfloat cRenderWorker::EnvMapping(const sShaderInputData &input)
 	dty = fmod(dty, texHeight);
 	if (dtx < 0) dtx = 0;
 	if (dty < 0) dty = 0;
-	envReflect.R = data->textures.envmapTexture.Pixel(dtx, dty).R / 256.0;
-	envReflect.G = data->textures.envmapTexture.Pixel(dtx, dty).G / 256.0;
-	envReflect.B = data->textures.envmapTexture.Pixel(dtx, dty).B / 256.0;
+
+	double reflectance = 1.0;
+	if (input.material->fresnelReflectance)
+	{
+		double n1 = 1.0;
+		double n2 = input.material ->transparencyIndexOfRefraction;
+		reflectance = Reflectance(input.normal, input.viewVector, n1, n2);
+		if (reflectance < 0.0) reflectance = 0.0;
+		if (reflectance > 1.0) reflectance = 1.0;
+	}
+
+	envReflect.R = data->textures.envmapTexture.Pixel(dtx, dty).R * reflectance;
+	envReflect.G = data->textures.envmapTexture.Pixel(dtx, dty).G * reflectance;
+	envReflect.B = data->textures.envmapTexture.Pixel(dtx, dty).B * reflectance;
 	return envReflect;
 }
 
@@ -889,14 +965,14 @@ sRGBAfloat cRenderWorker::SurfaceColour(const sShaderInputData &input)
 #else
 	sRGBAfloat out;
 
-	switch (input.objectType)
+	switch (data->objectData[input.objectId].objectType)
 	{
 		case fractal::objFractal:
 		{
 			sRGB colour(256, 256, 256);
-			if (params->coloringEnabled)
+			if (input.material->useColorsFromPalette)
 			{
-				int formulaIndex = input.formulaIndex;
+				int formulaIndex = input.objectId;
 
 				CVector3 tempPoint = input.point;
 
@@ -908,7 +984,7 @@ sRGBAfloat cRenderWorker::SurfaceColour(const sShaderInputData &input)
 					tempPoint *= params->formulaScale[formulaIndex];
 				}
 
-				sFractalIn fractIn(tempPoint, 0, params->N * 10, params->common, formulaIndex);
+				sFractalIn fractIn(tempPoint, 0, params->N * 10, params->common, formulaIndex, input.material->fractalColoring);
 				sFractalOut fractOut;
 				Compute<fractal::calcModeColouring>(*fractal, fractIn, &fractOut);
 				int nrCol = floor(fractOut.colorIndex);
@@ -921,10 +997,16 @@ sRGBAfloat cRenderWorker::SurfaceColour(const sShaderInputData &input)
 				}
 				else
 				{
-					color_number = (int) (nrCol * params->coloring_speed + 256 * params->paletteOffset)
+					color_number = (int) (nrCol * input.material->coloring_speed + 256 * input.material->paletteOffset)
 							% 65536;
 				}
-				colour = data->palette.IndexToColour(color_number);
+				colour = input.material->palette.IndexToColour(color_number);
+			}
+			else
+			{
+				colour.R = input.material->color.R / 256.0;
+				colour.G = input.material->color.G / 256.0;
+				colour.B = input.material->color.B / 256.0;
 			}
 
 			out.R = colour.R / 256.0;
@@ -943,9 +1025,9 @@ sRGBAfloat cRenderWorker::SurfaceColour(const sShaderInputData &input)
 		case fractal::objTorus:
 		case fractal::objCylinder:
 		{
-			out.R = input.objectColor.R / 65536.0;
-			out.G = input.objectColor.G / 65536.0;
-			out.B = input.objectColor.B / 65536.0;
+			out.R = input.material->color.R / 65536.0;
+			out.G = input.material->color.G / 65536.0;
+			out.B = input.material->color.B / 65536.0;
 			break;
 		}
 		case fractal::objNone:
@@ -959,12 +1041,12 @@ sRGBAfloat cRenderWorker::SurfaceColour(const sShaderInputData &input)
 #endif
 }
 
-sRGBAfloat cRenderWorker::LightShading(const sShaderInputData &input, cLights::sLight light,
+sRGBAfloat cRenderWorker::LightShading(const sShaderInputData &input, const cLights::sLight* light,
 		int number, sRGBAfloat *outSpecular)
 {
 	sRGBAfloat shading;
 
-	CVector3 d = light.position - input.point;
+	CVector3 d = light->position - input.point;
 
 	double distance = d.Length();
 
@@ -972,9 +1054,13 @@ sRGBAfloat cRenderWorker::LightShading(const sShaderInputData &input, cLights::s
 	CVector3 lightVector = d;
 	lightVector.Normalize();
 
-	double intensity = 100.0 * light.intensity / (distance * distance) / number;
+	// intensity of lights is divided by 6 because of backward compatibility. There was an error
+	// where number of light was always 24
+	double intensity = 100.0 * light->intensity / (distance * distance) / number / 6;
 	double shade = input.normal.Dot(lightVector);
 	if (shade < 0) shade = 0;
+	shade = (1.0 - input.material->shading) + shade * input.material->shading;
+
 	shade = shade * intensity;
 	if (shade > 500.0) shade = 500.0;
 
@@ -983,16 +1069,19 @@ sRGBAfloat cRenderWorker::LightShading(const sShaderInputData &input, cLights::s
 	half.Normalize();
 	double shade2 = input.normal.Dot(half);
 	if (shade2 < 0.0) shade2 = 0.0;
-	shade2 = pow(shade2, 30.0) * 1.0;
-	shade2 *= intensity * params->specular;
+
+	double diffuse = 10.0	* (1.1 - input.material->diffussionTextureIntensity	* (input.texDiffuse.R + input.texDiffuse.G + input.texDiffuse.B) / 3.0);
+
+	shade2 = pow(shade2, 30.0 / input.material->specularWidth / diffuse) / diffuse;
+	shade2 *= intensity * input.material->specular;
 	if (shade2 > 15.0) shade2 = 15.0;
 
 	//calculate shadow
 	if ((shade > 0.01 || shade2 > 0.01) && params->shadow)
 	{
-		double light = AuxShadow(input, distance, lightVector);
-		shade *= light;
-		shade2 *= light;
+		double auxShadow = AuxShadow(input, distance, lightVector);
+		shade *= auxShadow;
+		shade2 *= auxShadow;
 	}
 	else
 	{
@@ -1003,13 +1092,13 @@ sRGBAfloat cRenderWorker::LightShading(const sShaderInputData &input, cLights::s
 		}
 	}
 
-	shading.R = shade * light.colour.R / 65536.0;
-	shading.G = shade * light.colour.G / 65536.0;
-	shading.B = shade * light.colour.B / 65536.0;
+	shading.R = shade * light->colour.R / 65536.0;
+	shading.G = shade * light->colour.G / 65536.0;
+	shading.B = shade * light->colour.B / 65536.0;
 
-	outSpecular->R = shade2 * light.colour.R / 65536.0;
-	outSpecular->G = shade2 * light.colour.G / 65536.0;
-	outSpecular->B = shade2 * light.colour.B / 65536.0;
+	outSpecular->R = shade2 * light->colour.R / 65536.0;
+	outSpecular->G = shade2 * light->colour.G / 65536.0;
+	outSpecular->B = shade2 * light->colour.B / 65536.0;
 
 	return shading;
 }
@@ -1024,8 +1113,8 @@ sRGBAfloat cRenderWorker::AuxLightsShader(const sShaderInputData &input, sRGBAfl
 	sRGBAfloat specularAuxSum;
 	for (int i = 0; i < numberOfLights; i++)
 	{
-		cLights::sLight light = data->lights.GetLight(i);
-		if (i < params->auxLightNumber || light.enabled)
+		const cLights::sLight* light = data->lights.GetLight(i);
+		if (i < params->auxLightNumber || light->enabled)
 		{
 			sRGBAfloat specularAuxOutTemp;
 			sRGBAfloat shadeAux = LightShading(input, light, numberOfLights, &specularAuxOutTemp);
@@ -1105,7 +1194,7 @@ double cRenderWorker::AuxShadow(const sShaderInputData &input, double distance,
 double cRenderWorker::IterOpacity(const double step, double iters, double maxN, double trim,
 		double opacitySp)
 {
-	double opacity = ((double) iters - trim) / maxN;
+	double opacity = (iters - trim) / maxN;
 	if (opacity < 0.0) opacity = 0.0;
 	opacity *= opacity;
 	opacity *= step * opacitySp;
@@ -1168,7 +1257,8 @@ sRGBAfloat cRenderWorker::FakeLights(const sShaderInputData &input, sRGBAfloat *
 	half.Normalize();
 	double fakeSpecular = input.normal.Dot(half);
 	if (fakeSpecular < 0.0) fakeSpecular = 0.0;
-	fakeSpecular = pow(fakeSpecular, 30.0) * fakeLight;
+	double diffuse = 10.0	* (1.1 - input.material->diffussionTextureIntensity	* (input.texDiffuse.R + input.texDiffuse.G + input.texDiffuse.B) / 3.0);
+	fakeSpecular = pow(fakeSpecular, 30.0/input.material->specularWidth / diffuse) / diffuse;
 	if (fakeSpecular > 15.0) fakeSpecular = 15.0;
 	fakeSpec->R = fakeSpecular;
 	fakeSpec->G = fakeSpecular;
@@ -1178,3 +1268,116 @@ sRGBAfloat cRenderWorker::FakeLights(const sShaderInputData &input, sRGBAfloat *
 	return fakeLights;
 }
 
+sRGBfloat cRenderWorker::TextureShader(const sShaderInputData &input, cMaterial::enumTextureSelection texSelect, cMaterial *mat) const
+{
+	cObjectData objectData = data->objectData[input.objectId];
+	double texturePixelSize = 1.0;
+	CVector3 textureVectorX, textureVectorY;
+	CVector2<double> texPoint = TextureMapping(	input.point,
+																							input.normal,
+																							objectData,
+																							mat,
+																							&textureVectorX,
+																							&textureVectorY) + CVector2<double>(0.5, 0.5);
+
+	//mipmapping - calculation of texture pixel size
+	double delta = CalcDelta(input.point);
+	double deltaTexX = ((TextureMapping(input.point + textureVectorX * delta,
+																			input.normal,
+																			objectData,
+																			mat) + CVector2<double>(0.5, 0.5)) - texPoint).Length();
+	double deltaTexY = ((TextureMapping(input.point + textureVectorY * delta,
+																			input.normal,
+																			objectData,
+																			mat) + CVector2<double>(0.5, 0.5)) - texPoint).Length();
+
+	if(deltaTexX > 0.5) deltaTexX = 1.0 - deltaTexX;
+	if(deltaTexY > 0.5) deltaTexY = 1.0 - deltaTexY;
+
+  deltaTexX = fabs(deltaTexX) / fabs(input.viewVector.Dot(input.normal));
+  deltaTexY = fabs(deltaTexY) / fabs(input.viewVector.Dot(input.normal));
+
+	texturePixelSize = 1.0 / max(deltaTexX, deltaTexY);
+
+	sRGBfloat tex;
+	switch(texSelect)
+	{
+		case cMaterial::texColor:
+		{
+			tex = input.material->colorTexture.Pixel(texPoint, texturePixelSize);
+			break;
+		}
+		case cMaterial::texDiffuse:
+		{
+			tex = input.material->diffusionTexture.Pixel(texPoint, texturePixelSize);
+			break;
+		}
+		case cMaterial::texLuminosity:
+		{
+			tex = input.material->luminosityTexture.Pixel(texPoint, texturePixelSize);
+			break;
+		}
+		case cMaterial::texDisplacement:
+		{
+			tex = input.material->displacementTexture.Pixel(texPoint, texturePixelSize);
+			break;
+		}
+	}
+
+	return sRGBfloat(tex.R, tex.G, tex.B);
+}
+
+CVector3 cRenderWorker::NormalMapShader(const sShaderInputData &input)
+{
+	cObjectData objectData = data->objectData[input.objectId];
+	CVector3 texX, texY;
+	double texturePixelSize = 1.0;
+	CVector2<double> texPoint = TextureMapping(	input.point,
+																							input.normal,
+																							objectData,
+																							input.material,
+																							&texX,
+																							&texY) + CVector2<double>(0.5, 0.5);
+
+	//mipmapping - calculation of texture pixel size
+	double delta = CalcDelta(input.point);
+	double deltaTexX = ((TextureMapping(input.point + texX * delta,
+																			input.normal,
+																			objectData,
+																			input.material) + CVector2<double>(0.5, 0.5)) - texPoint).Length();
+	double deltaTexY = ((TextureMapping(input.point + texY * delta,
+																			input.normal,
+																			objectData,
+																			input.material) + CVector2<double>(0.5, 0.5)) - texPoint).Length();
+	deltaTexX = fabs(deltaTexX) / fabs(input.viewVector.Dot(input.normal));
+	deltaTexY = fabs(deltaTexY) / fabs(input.viewVector.Dot(input.normal));
+	texturePixelSize = 1.0 / max(deltaTexX, deltaTexY);
+
+	CVector3 n = input.normal;
+	//tangent vectors:
+	CVector3 t = n.Cross(texX);
+	t.Normalize();
+	CVector3 b = n.Cross(texY);
+	b.Normalize();
+	CMatrix33 tbn(b, t, n);
+
+	CVector3 tex;
+
+	if (input.material->normalMapTextureFromBumpmap)
+	{
+		tex = input.material->normalMapTexture.NormalMapFromBumpMap(texPoint,
+																																input.material
+																																		->normalMapTextureHeight,
+																																texturePixelSize);
+	}
+	else
+	{
+		tex = input.material->normalMapTexture.NormalMap(	texPoint,
+																											input.material->normalMapTextureHeight,
+																											texturePixelSize);
+	}
+
+	CVector3 result = tbn * tex;
+	result.Normalize();
+	return result;
+}
